/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Install = function() {
    var pub = {};

    //private properties
    //constants
    const NUM_INSTALL_SCREENS = 6;
    const QUICK_INSTALL_SCREEN = 7;
    const LANG_TO_SCREEN = 5;
    const HELPUS_INSTALL_SCREEN = 8;

    //private members
    var mCurrentScreen = 1;
    var mBeentoLangScreen = false;


    //public functions
    pub.openAreYouSureDialog = function() {
        var strbundle = document.getElementById("foxlingo-stringbundle");
        
        return window.confirm(strbundle.getString("foxlingo.text.quit.install"));
    };
    
    pub.prevScreen = function() {
        var thisScreenName;
        var prevScreenName;
        showNextButton(true);       
        if (mCurrentScreen==QUICK_INSTALL_SCREEN) {
            thisScreenName = QUICK_INSTALL_SCREEN;
            mCurrentScreen = 1;
        } else if (mCurrentScreen==HELPUS_INSTALL_SCREEN && document.getElementById("quickinstall").selected) {
            thisScreenName = HELPUS_INSTALL_SCREEN;
            mCurrentScreen = QUICK_INSTALL_SCREEN;
        } else if (mCurrentScreen==HELPUS_INSTALL_SCREEN) {
            thisScreenName = HELPUS_INSTALL_SCREEN;
            mCurrentScreen = NUM_INSTALL_SCREENS;
        } else {
            thisScreenName = mCurrentScreen;
            mCurrentScreen--;
        }
        var thisScreen = document.getElementById("screen-" + thisScreenName);
        var prevScreen = document.getElementById("screen-" + mCurrentScreen);

        thisScreen.collapsed = true;
        prevScreen.collapsed = false;
        
        if (mCurrentScreen==NUM_INSTALL_SCREENS-1)
            showNextButton(true);
        if (mCurrentScreen==1)
            showBackButton(false);
        var buttonsBox = document.getElementById("buttonsBox");
        buttonsBox.setAttribute("pack","end");
        buttonsBox = null;
    };
    
    pub.nextScreen = function() {
        showBackButton(true);
        
        var ADDIT_PREFS = Components.classes["@mozilla.org/preferences-service;1"].
                          getService(Components.interfaces.nsIPrefService).
                          getBranch("extensions." + FoxLingoAddItInstaller.ADDON_NAME +".addit.");
        var isShowHelpUs = ADDIT_PREFS.getBoolPref('isShowHelpUs');
        if (!isShowHelpUs) {
            if (mCurrentScreen==1 && document.getElementById("quickinstall").selected) {
                gotoScreen(QUICK_INSTALL_SCREEN);
                showNextButton(false);
                return;
            }
            
             if (mCurrentScreen==NUM_INSTALL_SCREENS-1) {
                gotoScreen(NUM_INSTALL_SCREENS);
                showNextButton(false);
                return;
            }
        }
        
        if (mCurrentScreen==1 && document.getElementById("quickinstall").selected) {
            gotoScreen(QUICK_INSTALL_SCREEN);
            return;
        }
        
        if (mCurrentScreen==QUICK_INSTALL_SCREEN || mCurrentScreen==NUM_INSTALL_SCREENS) {
            gotoScreen(HELPUS_INSTALL_SCREEN);
            var buttonsBox = document.getElementById("buttonsBox");
            buttonsBox.setAttribute("pack","center");
            buttonsBox = null;
            showNextButton(false);
            return;
        }
        
        
        var thisScreen = document.getElementById("screen-" + mCurrentScreen);
        mCurrentScreen++;
        var nextScreen = document.getElementById("screen-" + mCurrentScreen);
        
        if (!mBeentoLangScreen && mCurrentScreen==LANG_TO_SCREEN) {
            for (var language in Foxlingo.Languages) {
                if (Foxlingo.isAvailableLanguageTo(language))
                    Foxlingo.Options.initializeOption(language + "-langto"); 
            }
            mBeentoLangScreen = true;
        }
        
        thisScreen.collapsed = true;
        nextScreen.collapsed = false;
    };
    
    pub.finishedInstall = function() {
        var isFromMozilla = Foxlingo.Utils.getIsFromMozilla();
        var result = 0;
        if (isFromMozilla) {
        var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]  
        .getService(Components.interfaces.nsIPromptService);  
        var strbundle = document.getElementById("foxlingo-stringbundle");
        result = prompts.confirmEx(null, strbundle.getString("foxlingo.text.confirm.title"),
                                     strbundle.getString("foxlingo.text.confirm.siteLangIdentification"),
                                     1027 + 16777216,strbundle.getString("foxlingo.text.confirm.yes"), strbundle.getString("foxlingo.text.confirm.no"),"",null,{});  
        }
        Foxlingo.Prefs.setBoolPref("identifysitelanguage",(result == 0));
        pub.saveInstallOptions();
    };
    
    pub.saveInstallOptions = function(isOnCancel) {
        if (document.getElementById("quickinstall").selected && !isOnCancel)
            saveQuickInstallOptions();
        else // if this is a custom install
            saveCustomInstallOptions();  
        
        Foxlingo.Menus.createAllMenus(true);
        
        if (Foxlingo.Prefs.getBoolPref("changedlangto"))
            Foxlingo.Prefs.setBoolPref("onetranslatorperpair", false);
                
        Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).savePrefFile(null);

        var mainPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("browser.");
        //if (Foxlingo.Prefs.getCharPref("searchHomePage")) {
        //    mainPrefs.setCharPref("startup.homepage", "http://www.foxlingostart.com/");
        //    if (Foxlingo.Prefs.getCharPref("defaultsearchengine", true)=="zugoYahoo") {
        //        mainPrefs.setCharPref("startup.homepage", "http://www.foxlingo-start.com/");
        //    }
        //}
        if (Foxlingo.Prefs.getCharPref("searchFirefoxSearch")) {
            var currentEngineName = "Ask";
            if (Foxlingo.Prefs.getCharPref("defaultsearchengine", true)=="zugoBing")
                currentEngineName = "Bing";
            if (Foxlingo.Prefs.getCharPref("defaultsearchengine", true)=="zugoYahoo")
                currentEngineName = "Yahoo";
            var nsIBrowserSearchService = Components.classes["@mozilla.org/browser/search-service;1"].getService(Components.interfaces.nsIBrowserSearchService);
            var currentEngine = nsIBrowserSearchService.getEngineByName(currentEngineName);
            if (currentEngine) 
                nsIBrowserSearchService.removeEngine(currentEngine);
            nsIBrowserSearchService.addEngine("http://www.foxlingo.com/searchProviders/" + currentEngineName + ".xml", Components.interfaces.nsISearchEngine.DATA_XML, null, false);
            mainPrefs.setCharPref("search.selectedEngine", currentEngineName);
        }
    
        if (!isOnCancel)
            window.close();
    };
    
    pub.initializeAfterInstall = function() {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("foxlingo.");
        prefs.setCharPref("version", Foxlingo.Utils.getVersion());
    
        Foxlingo.Toolbar.updateToolbarButtons();
        Foxlingo.Toolbar.updateToolbarButtonsText();
        Foxlingo.Customize.applyStyles();    
        Foxlingo.Toolbar.setToolbar(true);
        Foxlingo.Locale.getSetLocale();
        Foxlingo.Utils.getClientIp();
        Foxlingo.Utils.getIsFromMozilla();
        if (Foxlingo.Prefs.getBoolPref("isShowInstaller")) {
            Foxlingo.Prefs.setBoolPref("isShowInstaller", false);
            // try getting the list. if no list after 60 seconds, show installer without last page.).
            Components.utils.import("resource://foxlingoadditinstaller/installer.js");
            var isDefaultsAddonsRetrieved = false;
            var ADDIT_PREFS = Components.classes["@mozilla.org/preferences-service;1"].
                           getService(Components.interfaces.nsIPrefService).
                           getBranch("extensions." + FoxLingoAddItInstaller.ADDON_NAME +".addit.");
            var prefType = ADDIT_PREFS.getPrefType('defaultAddonsRetrieved');
                if (prefType != ADDIT_PREFS.PREF_INVALID)
                    isDefaultsAddonsRetrieved =  ADDIT_PREFS.getBoolPref('defaultAddonsRetrieved');
            FoxLingoAddItInstallerExported.mIsDefaultsRetrieved.value = isDefaultsAddonsRetrieved;
            FoxLingoAddItInstaller.getAddonList();
            var counter = 1;
            var intervalHandle = window.setInterval(function() {
                                                    if (FoxLingoAddItInstallerExported.mIsDefaultsRetrieved.value && FoxLingoAddItInstallerExported.mAddonArray.value) {
                                                        Foxlingo.Utils.debug("have list, openning installer");
                                                        window.clearInterval(intervalHandle);
                                                        //installer with helpus
                                                        ADDIT_PREFS.setBoolPref('isShowHelpUs',true);
                                                        window.setTimeout(function() { Foxlingo.Toolbar.setToolbar(false); window.openDialog('chrome://foxlingo/content/install.xul', 'FoxLingoInstall','chrome,centerscreen,scrollbars=no,resizable=yes'); Foxlingo.initializeOnFirefoxLoad(true); }, 5000);
                                                        Foxlingo.Options.initializeInstallOptions();
                                                    } else {
                                                        Foxlingo.Utils.debug("don't have list, getting list");
                                                        FoxLingoAddItInstaller.getAddonList();
                                                    }
                                                }, 3000 * counter++);
            window.setTimeout(function() {
                Components.utils.import("resource://foxlingoadditinstaller/installer.js");
                if (!FoxLingoAddItInstallerExported.mIsDefaultsRetrieved.value) {
                    Foxlingo.Utils.debug("don't have list, 60 seconds passed... starting without helpus");
                    window.clearInterval(intervalHandle);
                    //installer without helpus
                    ADDIT_PREFS.setBoolPref('isShowHelpUs',false);
                    window.setTimeout(function() { Foxlingo.Toolbar.setToolbar(false); window.openDialog('chrome://foxlingo/content/install.xul', 'FoxLingoInstall','chrome,centerscreen,scrollbars=no,resizable=yes'); Foxlingo.initializeOnFirefoxLoad(true); }, 5000);
                    Foxlingo.Options.initializeInstallOptions();
                    }
                } ,80000);
        }
        if (Foxlingo.Prefs.getBoolPref("isShowAfterDownloadPage")) {
            Foxlingo.Utils.openAfterDownloadPage();
            Foxlingo.Prefs.setBoolPref("isShowAfterDownloadPage", false);
        }
    };
    

    //private functions
    function gotoScreen(screenNum) {
        var thisScreen = document.getElementById("screen-" + mCurrentScreen);
        var targetScreen = document.getElementById("screen-" + screenNum);
        
        mCurrentScreen = screenNum;

        thisScreen.collapsed = true;
        targetScreen.collapsed = false;
    }
    
    function quickInstall() {
        gotoScreen(QUICK_INSTALL_SCREEN);
        document.getElementById("nextbutton").collapsed = true;
        document.getElementById("finishbutton").collapsed = false;
        document.getElementById("backbutton").collapsed = false;
    }
    
    function saveQuickInstallOptions() {
        var engflag;
        var freflag;
        var spaflag;
        var defaultsearchengine;
        
        //if (document.getElementById("searchHomePageQI").checked)
        //    Foxlingo.Prefs.setBoolPref("searchHomePage", true);
        //if (document.getElementById("searchFirefoxSearchQI").checked)
        //    Foxlingo.Prefs.setBoolPref("searchFirefoxSearch", true);

        if (document.getElementById("engflagQI").selectedIndex===0)
            engflag = "usflag";
        else
            engflag = "ukflag";
        Foxlingo.Prefs.setCharPref("engflag", engflag);       
        if (document.getElementById("freflagQI").selectedIndex===0)
            engflag = "frflag";
        else
            engflag = "caflag";
        Foxlingo.Prefs.setCharPref("freflag", engflag);       
        if (document.getElementById("spaflagQI").selectedIndex===0)
            engflag = "spflag";
        else
            engflag = "meflag";
        Foxlingo.Prefs.setCharPref("spaflag", engflag);       
        Foxlingo.Prefs.setBoolPref("usestatusbar", document.getElementById("usestatusbarQI").checked);
        Foxlingo.Prefs.setCharPref("options-dlang", Foxlingo.Prefs.getCharPref("tempdlang")); 
        Foxlingo.Prefs.setCharPref("options-currency", Foxlingo.Prefs.getCharPref("tempcurrency")); 
    }

    function saveCustomInstallOptions() {  
        if (Foxlingo.Prefs.getBoolPref("changedlangto")) {
            for (var language in Foxlingo.Languages){
                if (Foxlingo.isAvailableLanguageTo(language))
                    Foxlingo.Options.saveOption(language + "-langto");
                if (Foxlingo.isAvailableLanguageFrom(language))
                    Foxlingo.Options.saveOption(language + "-langfrom");
            }
        }
        
        for each (var translator in Foxlingo.Translation.Translators)
            Foxlingo.Options.saveOption(translator);
            
        Foxlingo.Prefs.setCharPref("options-dlang", Foxlingo.Prefs.getCharPref("tempdlang")); 
        Foxlingo.Prefs.setCharPref("options-currency", Foxlingo.Prefs.getCharPref("tempcurrency")); 
        
        for each (var boolOption in Foxlingo.Options.BoolOptions)
            Foxlingo.Options.saveOption(boolOption);
            
        for (var boolRadioOptionsParent in Foxlingo.Options.BoolRadioOptionsParents) {
            for each (var boolRadioOption in Foxlingo.Options.BoolRadioOptionsParents[boolRadioOptionsParent]) {
                if (document.getElementById(boolRadioOption).getAttribute("selected")=="true")
                    Foxlingo.Options.saveOption(boolRadioOption, boolRadioOptionsParent, true);
            }
        }
        
        for each (var toolbarButton in Foxlingo.Toolbar.ToolbarButtons) {
            if (!document.getElementById('button-' + toolbarButton))
                continue;
                
            var prefValue = (document.getElementById('button-' + toolbarButton).getAttribute("checked")=="true");
            Foxlingo.Prefs.setBoolPref('button-' + toolbarButton, prefValue); 
        }
        
        Foxlingo.Options.ToolbarFlags.saveToolbarFlags('saveoptions');
        Foxlingo.Prefs.setCharPref("iconsizefl", Foxlingo.Toolbar.toolattr.getAttribute("iconsizefl"));
    }

    function showNextButton(isShow) {
        document.getElementById("nextbutton").collapsed = !isShow;
        document.getElementById("finishbutton").collapsed = isShow;
    }

    function showBackButton(isShow) {
        document.getElementById("backbutton").collapsed = !isShow;
    }


    return pub;
}();
