/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

var FoxlingoUninstallObserver = function() {
    var pub = {};
    var euid = "{ef62e1ce-d2a4-4cdd-b7ec-92b120366b66}";
    var afterUninstallSite = "http://www.foxlingo.com/uninstall/";
    var branch = "extensions.foxlingo.";

      pub.init = function() {
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(branch);
	var isUninstallFormShown = false;
	try {
	    isUninstallFormShown = prefs.getBoolPref("uninstallFormShown");
	} catch (e) {}
	if (isUninstallFormShown)
            return;
	prefs.setBoolPref("uninstallFormShown", true);
        Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService).addObserver(this, "em-action-requested", false);
	try {
	    //FF4
	    Components.utils.import("resource://gre/modules/AddonManager.jsm");
	    AddonManager.addAddonListener(this);
	} catch (e) {
	    
	}
    };
    
    pub.onUninstalling =  function(addon) {
	if (addon.id == euid) {
	    this.afterUninstallOperation(); 
	}
    }

    pub.observe = function(subject, topic, data) {
        try {
            if (topic!="em-action-requested")
                return;
            var updateItem = subject.QueryInterface(Components.interfaces.nsIUpdateItem);
            if (updateItem.id!=euid)
                return;
            if (data=="item-uninstalled") {
                this.afterUninstallOperation();
		return;
            }
        } catch (e) {}
	return;
    };

    pub.uninitAfterWindowUnload = function() {
        try {
            if (!gBrowser)
                return;
            FoxlingoUninstallObserver.uninit();
        } catch (e) {}
    }
    
    pub.afterUninstallOperation = function() {
	getBrowser().selectedTab = getBrowser().addTab(afterUninstallSite);
	return;
    }
    
    pub.uninit = function() {
	var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(branch);
	prefs.setBoolPref("uninstallFormShown", false);
        Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService).removeObserver(this, "em-action-requested");
    };

    return pub;
}();

var FoxlingoListener = function() {
    var pub = {};

    pub.QueryInterface = function(aIID) {
        if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
            aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
            aIID.equals(Components.interfaces.nsISupports))
            return this;
        throw Components.results.NS_NOINTERFACE;
    };

    pub.onLocationChange = function(aProgress, aRequest, aURI) {
        Foxlingo.processNewURL(aURI, aRequest);
        return 0;
    };

    pub.onProgressChange = function() { return 0; };
    pub.onStateChange = function() { return 0; };
    pub.onStatusChange = function() { return 0; };
    pub.onSecurityChange = function() { return 0; };
    pub.onLinkIconAvailable = function() { return 0; };

    return pub;
}();

var Foxlingo = function() {
    var pub = {};

    //public properties
    //constants
    pub.PREFS = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("foxlingo.");
    pub.isPBS = false;
    pub.pbsComponent = Components.classes["@mozilla.org/privatebrowsing;1"];
    if (pub.pbsComponent)
        pub.isPBS = pub.pbsComponent.getService(Components.interfaces.nsIPrivateBrowsingService).privateBrowsingEnabled;  

    pub.Languages = {
        auto: {shortName: 'xx', domain: 'com', numLangPairs: 50}, 
        english: {shortName: 'en', domain: 'com', numLangPairs: 225}, 
        afrikaans: {shortName: 'af', domain: 'za', numLangPairs: 52}, 
        albanian: {shortName: 'sq', domain: 'al', numLangPairs: 50}, 
        arabic: {shortName: 'ar', domain: 'eg', numLangPairs: 101}, 
        aranese: {shortName: 'oc', domain: 'es', numLangPairs: 2}, 
        armenian: {shortName: 'hy', domain: 'am', numLangPairs: 50}, 
        asturian: {shortName: 'as', domain: 'es', numLangPairs: 1}, 
        azerbaijani: {shortName: 'az', domain: 'az', numLangPairs: 50}, 
        basque: {shortName: 'eu', domain: 'es', numLangPairs: 1}, 
        belarusian: {shortName: 'be', domain: 'by', numLangPairs:50}, 
        bengali: {shortName: 'bn', domain: 'bd', numLangPairs:0}, 
        breton: {shortName: 'br', domain: 'fr', numLangPairs: 1}, 
        bulgarian: {shortName: 'bu', domain: 'bg', numLangPairs: 109}, 
        catalan: {shortName: 'ca', domain: 'cat', numLangPairs: 69}, 
        chineses: {shortName: 'cn', domain: 'cn', numLangPairs: 108, ixQuickLang: "jiantijongwen"}, 
        chineset: {shortName: 'tw', domain: 'tw', numLangPairs: 103, ixQuickLang: "fantizhengwen"}, 
        croatian: {shortName: 'cr', domain: 'hr', numLangPairs: 78}, 
        czech: {shortName: 'cz', domain: 'cz', numLangPairs: 128}, 
        danish: {shortName: 'da', domain: 'dk', numLangPairs: 133, ixQuickLang: "dansk"}, 
        dutch: {shortName: 'nl', domain: 'nl', numLangPairs: 132, ixQuickLang: "nederlands"}, 
        esperanto: {shortName: 'eo', domain: 'com', numLangPairs: 3}, 
        estonian: {shortName: 'et', domain: 'ee', numLangPairs: 50}, 
        filipino: {shortName: 'ph', domain: 'ph', numLangPairs: 78}, 
        finnish: {shortName: 'fi', domain: 'fi', numLangPairs: 110, ixQuickLang: "suomi"}, 
        french: {shortName: 'fr', domain: 'fr', numLangPairs: 164, ixQuickLang: "francais"}, 
        galician: {shortName: 'gl', domain: 'es', numLangPairs: 57}, 
        georgian: {shortName: 'ka', domain: 'ge', numLangPairs: 50}, 
        german: {shortName: 'de', domain: 'de', numLangPairs: 157, ixQuickLang: "deutsch"}, 
        greek: {shortName: 'el', domain: 'gr', numLangPairs: 130}, 
        haitiancreole: {shortName: 'ha', domain: 'ht', numLangPairs: 50}, 
        hausa: {shortName: 'ha', domain: 'ng', numLangPairs: 31}, 
        hebrew: {shortName: 'he', domain: 'il', numLangPairs: 111}, 
        hindi: {shortName: 'hi', domain: 'in', numLangPairs: 81}, 
        hungarian: {shortName: 'hu', domain: 'hu', numLangPairs: 110}, 
        icelandic: {shortName: 'ic', domain: 'is', numLangPairs: 79}, 
        indonesian: {shortName: 'in', domain: 'id', numLangPairs: 51}, 
        irish: {shortName: 'ir', domain: 'ie', numLangPairs:50}, 
        italian: {shortName: 'it', domain: 'it', numLangPairs: 146, ixQuickLang: "italiano"}, 
        japanese: {shortName: 'ja', domain: 'jp', numLangPairs: 152, ixQuickLang: "nihongo"}, 
        korean: {shortName: 'ko', domain: 'kr', numLangPairs: 106, ixQuickLang: "hangul"}, 
        latin: {shortName: 'la', domain: 'va', numLangPairs: 28}, 
        latvian: {shortName: 'lv', domain: 'lv', numLangPairs: 50}, 
        lithuanian: {shortName: 'lt', domain: 'lt', numLangPairs: 50}, 
        macedonian: {shortName: 'mk', domain: 'mk', numLangPairs: 53},  
        malay: {shortName: 'ma', domain: 'my', numLangPairs: 51},  
        maltese: {shortName: 'mt', domain: 'mt', numLangPairs: 50}, 
        nepali: {shortName: 'ne', domain: 'np', numLangPairs: 0}, 
        norwegian: {shortName: 'no', domain: 'no', numLangPairs: 113, ixQuickLang: "norsk"}, 
        norwegiann: {shortName: 'nn', domain: 'no', numLangPairs: 1}, 
        occitan: {shortName: 'oi', domain: 'es', numLangPairs: 2}, 
        pashto: {shortName: 'ps', domain: 'af', numLangPairs: 31}, 
        persian: {shortName: 'pe', domain: 'pe', numLangPairs: 32}, 
        polish: {shortName: 'po', domain: 'pl', numLangPairs: 134, ixQuickLang: "polski"}, 
        portuguese: {shortName: 'pt', domain: 'pt', numLangPairs: 146, ixQuickLang: "portugues"}, 
        portugueseb: {shortName: 'pb', domain: 'br', numLangPairs: 28}, 
        romanian: {shortName: 'ro', domain: 'ro', numLangPairs: 111}, 
        russian: {shortName: 'ru', domain: 'ru', numLangPairs: 144}, 
        scottishg: {shortName: 'gd', domain: 'uk', numLangPairs: 1}, 
        serbian: {shortName: 'se', domain: 'rs', numLangPairs: 109}, 
        serbocroatian: {shortName: 'sc', domain: 'ba', numLangPairs: 1}, 
        serbocroatians: {shortName: 'sb', domain: 'ba', numLangPairs: 0}, 
        serbocroatianz: {shortName: 'sh', domain: 'hr', numLangPairs: 0}, 
        serbocroatianb: {shortName: 'ss', domain: 'rs', numLangPairs: 0}, 
        slovak: {shortName: 'sk', domain: 'sk', numLangPairs: 50}, 
        slovenian: {shortName: 'sl', domain: 'si', numLangPairs: 78}, 
        somali: {shortName: 'so', domain: 'so', numLangPairs: 32}, 
        spanish: {shortName: 'es', domain: 'es', numLangPairs: 177, ixQuickLang: "espanol"}, 
        spanishla: {shortName: 'sa', domain: 'es', numLangPairs: 28}, 
        swahili: {shortName: 'sw', domain: 'ke', numLangPairs: 52}, 
        swedish: {shortName: 'sv', domain: 'se', numLangPairs: 132, ixQuickLang: "svenska"}, 
        tajik: {shortName: 'tg', domain: 'tj', numLangPairs: 1}, 
        thai: {shortName: 'th', domain: 'th', numLangPairs: 101}, 
        turkish: {shortName: 'tu', domain: 'tr', numLangPairs: 109, ixQuickLang: "turkce"}, 
        ukrainian: {shortName: 'uk', domain: 'ua', numLangPairs: 50}, 
        urdu: {shortName: 'ur', domain: 'pk', numLangPairs: 31}, 
        valencian: {shortName: 'va', domain: 'cat', numLangPairs: 0}, 
        vietnamese: {shortName: 'vi', domain: 'vn', numLangPairs: 50}, 
        welsh: {shortName: 'we', domain: 'uk', numLangPairs: 79}, 
        yiddish: {shortName: 'yi', domain: 'com', numLangPairs: 50} 
    };

pub.Currencies = {
        aud: {},
        brl: {},
        cad: {},
        cny: {},
        chf: {},
        eur: {},
        gbp: {},
        ils: {},
        jpy: {},
        usd: {}
    };
    
    //public members
    pub.session;
    pub.mainWindow;
    pub.tabBrowser;
    pub.direction = "ltr";

    //private members
    var mIsGlobalInitialized = false;
    var mTempUrl = null;
    var mOldUrl = null;

    //public functions
    pub.init = function() {
        pub.mainWindow = Foxlingo.Utils.mainWindow();
        pub.session = Foxlingo.Utils.getSession();
        pub.session.cache = {};

        if (!pub.session.currentDomainButtonProperties)
            pub.session.currentDomainButtonProperties = {};
        window.addEventListener("DOMContentLoaded", hoverInit, true);
        window.addEventListener("load", mainloop, false);
        window.addEventListener("unload", uninitAfterWindowUnload, false);        
        FoxlingoUninstallObserver.init();
        try {
            pub.session.cache["tabSelected"] = "";
            if (!gBrowser)
                return;
            gBrowser.tabContainer.addEventListener("TabSelect", tabSelected, false);
        } catch (e) { }
    };

    pub.getTabBrowser = function() {
        if (!pub.tabBrowser)
            pub.tabBrowser = document.getElementById("content");
        return pub.tabBrowser;
    };

    pub.initializeOnFirefoxLoad = function(afterInstall) {
        if (!document.getElementById('Foxlingo_mainmenu-item'))
            Foxlingo.Utils.debug("no mainmenu");
        Foxlingo.session.emptyMenuList = "";
        window.document.addEventListener("DOMContentLoaded", function(event) {Foxlingo.Utils.insertWorldLingoAds(event);}, true);

        pub.tabBrowser = document.getElementById("content");
        
        Foxlingo.Menus.buildFlagsOverflowMenu();
        Foxlingo.Toolbar.Flags.removeAllChildrenFromToolbarFlags();
        Foxlingo.Toolbar.updateToolbarButtons();
        Foxlingo.Toolbar.updateToolbarButtonsText();
    
        Foxlingo.Menus.createAllMenus(true);        
        Foxlingo.Search.Geo.initGeoCombinedSearch();
        Foxlingo.Search.Dictionary.updatePromtIcon();
        Foxlingo.Search.Translated.updateTranslatedSearchLangTo(Foxlingo.Prefs.getCharPref("translatedsearchlangto"), false);
        Foxlingo.Search.Translated.updateTranslatedSearchLangFrom(Foxlingo.Prefs.getCharPref("translatedsearchlangfrom"), false);
        Foxlingo.Search.loadSearchHistory();
    
        if (!pub.session.isChangedSearchboxSize) {
            window.setTimeout(function() { var searchBox = document.getElementById('Foxlingo_searchbox-item'); 
                                                           document.persist('Foxlingo_searchbox-item', 'width'); 
                                                         }, 200);
        }
        
        window.setTimeout(function() { Foxlingo.Toolbar.resize(); }, 700);    
        var resizerItem = document.getElementById("Foxlingo_Search-ResizeSplitter");
        if (resizerItem)
            resizerItem.dragging = false;       
        
        Foxlingo.Customize.applyStyles();
        
        window.addEventListener("click", function(event) {Foxlingo.globalMouseClick(event)}, false);
        window.addEventListener("mouseup", globalMouseUp, true);
        window.addEventListener("mousemove", pub.globalMouseMove, true);
        document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", initContext, true);
                
        var tabBrowserHandler = pub.getTabBrowser().mTabBox;
        tabBrowserHandler.addEventListener("select", onSelect, true);
        tabBrowserHandler.addEventListener("pageshow", onPageShow, false);
        tabBrowserHandler.addEventListener("resize", Foxlingo.Toolbar.doResize, true);
        
        pub.direction = window.getComputedStyle(pub.tabBrowser, "").direction;
        
        if (pub.direction=="rtl") {
        }
        
        Foxlingo.Search.Translated.setTranslatedSearchIcon();    
        
        Foxlingo.Locale.getSetLocale();
        var countryFound = getClientFFLocaleCountry();
        var ixQuickDefaultLang = "english";
        
        if (countryFound) {
            for each (var language in pub.Languages) {
                if (language.shortName==countryFound && language.ixQuickLang)
                    ixQuickDefaultLang = language.ixQuickLang;
            }
        }
        pub.session.ixquickDefaultLang = ixQuickDefaultLang;
    };

    pub.openSite = function(service, fromtextbox, returnValue, target) {
        var selection = '';
        
        if (!fromtextbox) {
            selection = content.window.getSelection().toString(); 
            selection = Foxlingo.Utils.stripJunkText(selection);
        } else if (fromtextbox=='textbox') {
            if (Foxlingo.Prefs.getBoolPref("translatedsearch")) {
                Foxlingo.Translation.openSiteTranslated(service);
                return false;
            }
        } else
            selection = fromtextbox;
        if (document.getElementById("foxlingo-search-box"))
            selection = document.getElementById("foxlingo-search-box").value;
        selection = encodeURIComponent(selection);
        
        var openTabUrl;
        if (!selection || !Foxlingo.Service.doesServiceUseSpecialSelection(service)) {
            var url = Foxlingo.Utils.getUrl();
            if (url!="about:blank")
                openTabUrl = Foxlingo.Service.getNonBlankServiceUrl(service, url);
            if (!openTabUrl)    
                openTabUrl = Foxlingo.Service.getServiceUrl(service);
        } else {
            openTabUrl = Foxlingo.Service.getServiceSelectionUrl(service, selection);
        }
        
        if (openTabUrl && openTabUrl.indexOf('www.degraeve.com/cgi-bin/babel.cgi')!=-1)
            openTabUrl = openTabUrl.split('%23')[0];
                
        if (returnValue)
            return openTabUrl;
            
        if (target)
            target.location.href = openTabUrl;
        else if (fromtextbox) 
            Foxlingo.Utils.openTabS(openTabUrl);
        else
            Foxlingo.Utils.openTabL(openTabUrl);
        return true;
    };
    
    pub.backandDomain = function() {
        var isAutoTranslate = Foxlingo.Prefs.getBoolPref("autotranslate");
        
        if (Foxlingo.Translation.isUrlTranslated()) {
            mTempUrl = Foxlingo.Translation.backToOriginal(); 
            Foxlingo.Translation.Auto.backToOriginalList.push(mTempUrl);
            mTempUrl = decodeURIComponent(mTempUrl);
            window.content.document.location.href = mTempUrl;
            return;
        }
        
        if (isAutoTranslate) {
            Foxlingo.Translation.Auto.findRemoveBackList(window.content.document.URL);
            Foxlingo.Translation.Auto.findRemoveSentList(window.content.document.URL);
        }
        var langto = Foxlingo.Prefs.getCharPref("options-dlang");
        var langfrom = Foxlingo.Utils.getDomain("language");
        
        mTempUrl = '';
        Foxlingo.Translation.Auto.autoTranslate(langfrom, langto, Foxlingo.Translation.getTranslator(langfrom, langto), "button");
    };
    
    pub.globalMouseClick = function(event) {
        if (event.target && event.button==2) {
            if (Foxlingo.Prefs.getBoolPref("shiftKey") && event.shiftKey) {
                event.preventDefault();
                Foxlingo.Translation.Auto.autotranslateFrom('', true, event);
            } else if (Foxlingo.Prefs.getBoolPref("ctrlKey") && event.ctrlKey) {
                event.preventDefault();
                Foxlingo.Utils.openDictionary(event, true);
            }
        }
    }

    pub.globalMouseMove = function(event, checkResizer) {
        var xDiff = 0;
        
        if (!checkResizer) {
            if (!Foxlingo.session.dragging)
                return;
            xDiff = Foxlingo.session.lastX - event.screenX;
        }
        
        var searchboxItem = document.getElementById('Foxlingo_searchbox-item');
        if (!searchboxItem) 
            return;
        if (!searchboxItem.boxObject)
            return;
        var computedWidth = parseInt(searchboxItem.boxObject.width, 10);
        var xOffset = searchboxItem.boxObject.x;
        var available = window.innerWidth;
        var translatedsearch = document.getElementById("Foxlingo_translatedsearch-item");
        var highlighter = document.getElementById("Foxlingo_highlighter-item");
        var searchwordbuttons = document.getElementById("Foxlingo_searchwordbuttons-item");
        var searchwordoverflowButton = document.getElementById("Foxlingo_Overflow-Button");
        var searchItem = document.getElementById('Foxlingo_search-item');
        
        var searchwordoverflowButtonWidth = 0;
        if (searchwordoverflowButton)
            searchwordoverflowButtonWidth = searchwordoverflowButton.boxObject.width;
        
        var translatedsearchWidth = 0;
        if (translatedsearch && translatedsearch.boxObject)
            translatedsearchWidth = translatedsearch.boxObject.width;
                
        var highlighterWidth = 0;
        if (highlighter && highlighter.boxObject)
            highlighterWidth = highlighter.boxObject.width;
                
        var searchwordbuttonsWidth = 0;
        if (searchwordbuttons && searchwordbuttons.boxObject) {
            searchwordbuttonsWidth = searchwordbuttons.boxObject.width;
            if (searchwordbuttonsWidth > 20) 
                searchwordbuttonsWidth = 20;
        }
        
        var searchItemWidth = 0;
        if (searchItem && searchItem.boxObject)
            searchItemWidth = searchItem.boxObject.width;
    
        var toolbarflagsareaWidth = 0;
        if (Foxlingo.Prefs.getBoolPref("toolbarflags")) {
            for (var language in pub.Languages) {
                if (!pub.isAvailableLanguageFrom(language))
                    continue;
                
                var flag = document.getElementById("Foxlingo_" + language + "-item");
                if (flag && flag.boxObject)
                    toolbarflagsareaWidth += flag.boxObject.width;
            }
            
            if (toolbarflagsareaWidth > 20) 
                toolbarflagsareaWidth = 20;
        }
        
        var resizeLimit;
        if (pub.direction=='ltr')
            resizeLimit = available - xOffset;
        else
            resizeLimit = xOffset + computedWidth;
        resizeLimit -= searchItemWidth + translatedsearchWidth + highlighterWidth + searchwordbuttonsWidth + searchwordoverflowButtonWidth + toolbarflagsareaWidth;
        
        if (computedWidth > resizeLimit) {
            computedWidth = resizeLimit;
        } 
        
        if (pub.direction=="ltr")
            computedWidth -= xDiff;
        else
            computedWidth += xDiff;
        
        if (computedWidth > 0)
            searchboxItem.setAttribute("width", computedWidth);
        
        if (!checkResizer)
            Foxlingo.session.lastX = event.screenX;
    };
    
    pub.isAvailableLanguageFrom = function(lang) {
          var noDefaultLangs = ['bengali', 'nepali', 'valencian', 'serbocroatians', 'serbocroatianz', 'serbocroatianb'];
          return (lang && noDefaultLangs.indexOf(lang)==-1);
    };
    
    pub.isAvailableLanguageTo = function(lang) {
          var noDefaultLangs = ['auto', 'basque', 'breton', 'scottishg', 'serbocroatian'];
          return (lang && noDefaultLangs.indexOf(lang)==-1);
    };

    pub.handleOnSelect = function() {
        var curDoc = Foxlingo.Utils.getCurrentDocument();
        if (curDoc)
            Foxlingo.Status.displayTabStatus(curDoc.URL);
    };

    pub.processNewURL = function(browserUrl, request) {
        var url = "about:blank"
        if (browserUrl && browserUrl!=null) {
            if (browserUrl.spec==this.mOldUrl)
                return;
            else 
                url = browserUrl.spec;
        }
        this.mOldUrl = url;
        if (!Foxlingo.Prefs.getBoolPref("searchBoxCapturing"))
            Foxlingo.Search.resetSearchBox(url);
        var referer = "";
        if(request && request.referrer!=null && typeof(request.referrer)!="undefined")
            referer = request.referrer.spec;
        Foxlingo.Toolbar.changeDomainButton(null, referer);
    }

    pub.hoverInitActual = function(event) {
        try {
            if (!gContextMenu || gContextMenu && gContextMenu.linkURL)
                document.getElementById("Foxlingo_context-dictionary-item").word = "";
            if (event.rangeParent && event.rangeParent.nodeType==Node.TEXT_NODE
             && event.rangeParent.textContent && event.rangeParent.textContent.length
             && content && content.window && content.window.getSelection() && content.window.getSelection().toString().length<1
             && (!gContextMenu || (gContextMenu && !gContextMenu.linkURL))) {
                var word = Foxlingo.Utils.parseWord(event.rangeParent.textContent, event.rangeOffset);
                if (!word)
                    word = "";
                var cutWord = word.substr(0, 25)
                if (cutWord!=word)
                    word = cutWord + "...";
                document.getElementById("Foxlingo_context-dictionary-item").label = document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.dictionaryWikipedia") + ": " + document.getElementById("Foxlingo_context-dictionary-item").word;
                document.getElementById("Foxlingo_context-dictionary-item").word = word;
            }
        } catch(e) {}
    }
   
    //private functions
    function tabSelected() {
        pub.session.cache["tabSelected"] = "true";
    }

    function hoverInit(event) {
	event.originalTarget.addEventListener("mousemove", pub.hoverInitActual, true);
        if (event.originalTarget instanceof HTMLDocument) {
            var doc = event.originalTarget;
                if (event.originalTarget.defaultView.frameElement) {
                    if (event.originalTarget.defaultView.frameElement.src &&
                        event.originalTarget.defaultView.frameElement.src.indexOf("https://www.superfish.com/ws/index.jsp")==0) {
                        event.originalTarget.defaultView.frameElement.contentWindow.addEventListener("DOMNodeInserted",
                            function(event) {
                                if (event.originalTarget && event.originalTarget.nodeName=="TABLE") {
                                    var child = event.originalTarget.firstChild.firstChild;
                                    var textElements = child.getElementsByClassName('title');
                                    var textToTranslate = "";
                                    var separator = " *** ";
                                    for (var element in textElements) {
                                        if (textElements[element].innerHTML)
                                            textToTranslate += textElements[element].innerHTML + separator;
                                    }
                                    var injectionProperties = {"text":textToTranslate,"elements":textElements,"separator":separator};
                                    Foxlingo.Search.Translated.translatedText(injectionProperties);
                                }
                            },true);
                    }
                }
        }
    }

    function mainloop() {
        if (!Foxlingo.Utils.isBrowserWindow())
            return;
        Foxlingo.Search.resetSearchBox();
        if (!mIsGlobalInitialized) {
            initializeFoxLingo();
            mIsGlobalInitialized = true;
        }
    }
   
    function uninitAfterWindowUnload() {
        try {
            if (!gBrowser)
                return;
            gBrowser.removeProgressListener(FoxlingoListener);
            FoxlingoUninstallObserver.uninit();
        } catch (e) {}
    }
       
    function getClientFFLocaleCountry() {
        var countryFound = '';
        var clientFFLocale = Foxlingo.Prefs.getCharPref("clientfflocale");
        if (clientFFLocale) {
            clientFFLocale = clientFFLocale.split("-");
            if (clientFFLocale.length==2 && clientFFLocale[0]!="en" && clientFFLocale[0]!="undefined")
                countryFound = clientFFLocale[0];
        }
        if (!countryFound)
            countryFound = Foxlingo.Prefs.getCharPref("foundipcountry");
        return countryFound;
    }

    function insertToggleToolbarButton() {
        try {
            var firefoxnav = document.getElementById("nav-bar");
            var curSet = firefoxnav.currentSet;
            if (curSet.indexOf("Foxlingo_toggletoolbar-item") == -1) {
                var set;
                // Place the button before the urlbar
                if (curSet.indexOf("urlbar-container") != -1)
                    set = curSet.replace(/urlbar-container/, "Foxlingo_toggletoolbar-item,urlbar-container");
                else  // at the end
                    set = curSet + ",Foxlingo_toggletoolbar-item";
                firefoxnav.setAttribute("currentset", set);
                firefoxnav.currentSet = set;
                document.persist("nav-bar", "currentset");
                // If you don't do the following call, funny things happen
                try {
                     BrowserToolboxCustomizeDone(true);
                } catch (e) { }
            }
        }
        catch(e) { }
    }

    function initializeFoxLingo() {
        if (!Foxlingo.Utils.isBrowserWindow())
            return;
        gBrowser.addProgressListener(FoxlingoListener, Components.interfaces.nsIWebProgress.NOTIFY_STATE_DOCUMENT);
        var prefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("foxlingo.");
        var version;
        if (prefs.prefHasUserValue("version"))
            version = prefs.getCharPref("version");
        if (!version)
            version = "0.0";
        version = version.replace(".", "-");
        var newVersion = Foxlingo.Utils.getVersion().replace(".", "-");
        version = parseFloat(version.replace(/\./g, "").replace("-", "."));
        newVersion = parseFloat(newVersion.replace(/\./g, "").replace("-", "."));
        if (version < 2) {
            insertToggleToolbarButton();
            Foxlingo.Install.initializeAfterInstall(); 
        } else if (version < newVersion) {                
            if (newVersion.toString()=="2.69") {
                insertToggleToolbarButton();
                Foxlingo.Utils.setCurrencyAccordingToCountry();
            }
            prefs.setCharPref("version", Foxlingo.Utils.getVersion());   
            window.setTimeout(function() { Foxlingo.Utils.openUrlFromDialog("http://www.foxlingo.com/update/?v=" + Foxlingo.Utils.getVersion(), Foxlingo.Utils.getStatsReferer()); }, 5000);
            pub.initializeOnFirefoxLoad(true);
            // move all existing non-zugoBing && non-zugoYahoo US users to zugoYahoo
            var country = Foxlingo.Prefs.getCharPref("foundipcountry");
            var provider = "ixquick";
            if (prefs.getCharPref("defaultsearchengine")!= "zugoBing")
                prefs.setCharPref("defaultsearchengine", provider);
        } else {
            pub.initializeOnFirefoxLoad(false);
        }
        mIsGlobalInitialized = true;
        //if (!document.getElementById("foxlingo-toolbar-" + Foxlingo.Toolbar.TOOLBAR_VERSION).collapsed)
        //    Foxlingo.Utils.trackToolbar("{ef62e1ce-d2a4-4cdd-b7ec-92b120366b66}");
    }

    function onSelect(event) { 
       window.setTimeout(function() { Foxlingo.handleOnSelect(); }, 0); 
    }
    
    function initContext(event) {
       try {
            var isUseContextMenu = Foxlingo.Prefs.getBoolPref("usecontextmenu");   
            var selection = content.window.getSelection().toString();
            var wm = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
            var win = wm.getMostRecentWindow("navigator:browser");
            // Web Translation
            if (isUseContextMenu && gContextMenu && !gContextMenu.linkURL
               && window.content && window.content.location && Foxlingo.Utils.isWebUrl(window.content.location.href.toString()))
                document.getElementById("Foxlingo_context-webtranslation-item").hidden = false;
            else
                document.getElementById("Foxlingo_context-webtranslation-item").hidden = true;
            // Text Translation
            if (isUseContextMenu && ((selection.length!=0 && selection.split(' ').length>1)
                                   || (selection.length==0 && gContextMenu.target && gContextMenu.target.textContent && gContextMenu.target.textContent.toString().length!=0
                                      && gContextMenu.target.textContent && gContextMenu.target.textContent.toString().length!=0
                                      && gContextMenu.target.textContent.split(' ').length>2)))
                document.getElementById("Foxlingo_context-texttranslation-item").hidden = false;
            else
                document.getElementById("Foxlingo_context-texttranslation-item").hidden = true;
            // Dictionary
            if (isUseContextMenu && (((selection.length!=0 && selection.split(' ').length<20)
                                   || (gContextMenu.target && gContextMenu.target.textContent
                                      && gContextMenu.target.textContent && gContextMenu.target.textContent.toString().length!=0
                                      && gContextMenu.target.textContent.split(' ').length<10))
                                   || (win.document.getElementById("Foxlingo_context-dictionary-item")
                                      && win.document.getElementById("Foxlingo_context-dictionary-item").word!="")))
                document.getElementById("Foxlingo_context-dictionary-item").hidden = false;
            else
                document.getElementById("Foxlingo_context-dictionary-item").hidden = true;
            // Set Dictionary Label
            var defaultLabel = document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.dictionaryWikipedia");
            var dictionaryElement = document.getElementById("Foxlingo_context-dictionary-item");
            if (selection.length!=0 && selection.split(' ').length<10)
                dictionaryElement.label = defaultLabel + ": " + selection;
            else if (gContextMenu && gContextMenu.target && gContextMenu.target.textContent && gContextMenu.linkURL)
                dictionaryElement.label = defaultLabel + ": " + gContextMenu.target.textContent;
            else if (win.document.getElementById("Foxlingo_context-dictionary-item") && win.document.getElementById("Foxlingo_context-dictionary-item").word)
                dictionaryElement.label = defaultLabel + ": " + win.document.getElementById("Foxlingo_context-dictionary-item").word;
            else if (gContextMenu && gContextMenu.target && gContextMenu.target.textContent)
                dictionaryElement.label = defaultLabel + ": " + gContextMenu.target.textContent;
            else if (win.document.getElementById("Foxlingo_context-dictionary-item"))
                dictionaryElement.label = defaultLabel;
        } catch (e) {}
    }

    function globalMouseUp(event) {
        try {
            if (Foxlingo.session.dragging) {
                Foxlingo.session.dragging = false;
                Foxlingo.Toolbar.resize(true);
            }
            // search box grab selection
            if (!event.originalTarget || !event.originalTarget.ownerDocument || !event.originalTarget.ownerDocument.activeElement
                || event.originalTarget.ownerDocument.activeElement.nodeName!="BODY")
                return;
            var selection = content.window.getSelection().toString();
            if (selection && !Foxlingo.Prefs.getBoolPref("searchBoxCapturing"))
                Foxlingo.Search.resetSearchBox("", selection.substr(0, 1300));
        } catch (e) {}
    }
    
    function onPageShow(event, target, fromCDB) {
        var eventTarget;

        if (event)
            eventTarget = event.target;
        else if (target)
            eventTarget = target;
        else
            eventTarget = Foxlingo.Utils.getCurrentDocument();
    
        if (!Foxlingo.Utils.getTabByUrl(eventTarget.URL) || eventTarget.URL=="about:blank")
        return false;
        
        var langTo = Foxlingo.Prefs.getCharPref("options-dlang");
        
        if (Foxlingo.Prefs.getBoolPref("autotranslate")) {
            if (Foxlingo.Translation.Auto.findBackList(eventTarget.URL) || Foxlingo.Translation.isUrlTranslated(eventTarget) ||
                Foxlingo.Translation.Auto.findRemoveSentList(eventTarget.URL))
                return true;
                
            Foxlingo.Translation.Auto.alreadySentList.push(eventTarget.URL);
            var langFrom = Foxlingo.Utils.getDomain("language", null, eventTarget);
            var translator = Foxlingo.Translation.getTranslator(langFrom, langTo);
            
            var tempDomain = Foxlingo.Utils.getDomain("domain", null, eventTarget);
            if (tempDomain=="unknowndomain" && translator=="google") {
                Foxlingo.Translation.Auto.autoTranslate("auto", langTo, translator, null, eventTarget);
            } else if (tempDomain!="unknowndomain" && translator!="none") {
                Foxlingo.Translation.Auto.autoTranslate(langFrom, langTo, translator, null, eventTarget);
            }
            return true;
        }
        
        if (!document.getElementById("Foxlingo_backanddomain-item") || !Foxlingo.Prefs.getBoolPref("button-backanddomain"))
            return false;
            
        return true;
    }

    return pub;
    //} catch (e) {
    //Foxlingo.Utils.debugException(e);
    //}
}();
