/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Options = function() {
    var pub = {};

    //public properties
    //constants
    pub.BoolOptions = ['toggleall-langto', 'toggleall-langfrom', 'toggleall-translators', 'onetranslatorperpair', 'button-services',
                       'button-texttranslation', 'button-backanddomain', 'button-grammar', 'button-dictionary', 'button-translatedsearch', 'button-options', 'button-highlighter',
                       'button-searchwordbuttons', 'fulllogo', 'toolbarflags', 'autotranslate', 'identifysitelanguage', 
                       'showbuttontext', 'usetoolbar', 'usecontextmenu', 'usestatusbar', 'shiftKey', 'ctrlKey', 'clickablewebtrans','enabledPriceComparison',
                       'downloadWhitesmoke', 'searchBoxCapturing', 'autotransWorldlingo']; //'searchHomePage', 'searchFirefoxSearch']; 
    pub.BoolRadioOptionsParents = {
        engflag: ['usflag', 'ukflag'],
        freflag: ['frflag', 'caflag'],
        spaflag: ['spflag', 'meflag'],
        opent: ['openCurrentT', 'openTabT', 'openWindowT'],
        openl: ['openCurrentL', 'openTabL', 'openWindowL'],
        opens: ['openCurrentS', 'openTabS', 'openWindowS']
    }; 

    //public members
    pub.changedMenuOptions = false;


    //public functions
    pub.toggleAll = function(type) {
        if (type=="translators") {
            for each (var translator in Foxlingo.Translation.Translators) {
                document.getElementById(translator).checked = document.getElementById("toggleall-translators").checked;
            }
            return;
        }
        
        for (var language in Foxlingo.Languages) {
            if (type=="langto" && Foxlingo.isAvailableLanguageTo(language)) {
                document.getElementById(language + "-langto").checked = document.getElementById("toggleall-langto").checked;
            } else if (Foxlingo.isAvailableLanguageFrom(language)) {
                if (type=="langfrom") {
                    document.getElementById(language + "-langfrom").checked = document.getElementById("toggleall-langfrom").checked;                        
                    if (Foxlingo.Prefs.getCharPref('firsttimetoolbarflags')!='falsefinal') {
                        Foxlingo.session[language + "-toolbartemp"] = document.getElementById("toggleall-langfrom").checked;
                        Foxlingo.Toolbar.Flags.populateToolbarFlags(true);    
                    }
                } else if (type=="toolbarflags") {
                    document.getElementById("toolbarflag-" + language).checked = document.getElementById("toggleall-toolbarflags").checked;
                    Foxlingo.Toolbar.Flags.toggleToolbarFlag(language, document.getElementById("toggleall-toolbarflags").checked);
                }
            }
        }
    };
    
    pub.toggleOption = function(id) {
        var option = document.getElementById(id);    
        var optValue;

        if (option.getAttribute("disabled")=="true")
            optValue = "false";
        else
            optValue = "true";

        option.setAttribute("disabled", optValue);
        document.persist(id, "disabled");
    };

    pub.setOption = function(id, boolVal) {
        var option = document.getElementById(id);    
        option.setAttribute("disabled", boolVal);
        document.persist(id, "disabled");
    };
    
    pub.updateToggleAllButton = function(type) {
        document.getElementById("toggleall-" + type).checked = isAllToggled(type, true);
    };
    
    pub.initializeInstallLangtoOptions = function() {
        createOptions('langto', 20);
        
        for (var language in Foxlingo.Languages) { 
            if (Foxlingo.isAvailableLanguageTo(language)) {
                pub.initializeOption(language + "-langto");
            }
        }
        pub.updateToggleAllButton('langto');
    };
    
    pub.saveInstallLangtoOptions = function() {
        for (var language in Foxlingo.Languages) { 
            if (Foxlingo.isAvailableLanguageTo(language))
                Foxlingo.Prefs.setBoolPref(language + '-langto', document.getElementById(language + '-langto').checked);
            
        }
    };
    
    pub.initializeInstallOptions = function() {
        var country = Foxlingo.Prefs.getCharPref("foundipcountry");
        var localeToSet;
        var countryToSet;
        
        var oPrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranchInternal);
        var oDefPref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getDefaultBranch("general.useragent.");
        
        if (Foxlingo.Utils.isFF) {
            var ffDefaultLocale = oDefPref.getCharPref("locale").split('-')[0];
            var ffLocale = oPrefService.getCharPref("general.useragent.locale").split('-')[0];
            localeToSet = Foxlingo.Locale.changeFFLocaleCode(ffLocale);
            countryToSet = Foxlingo.Locale.changeFFLocaleCode(ffDefaultLocale);
        } else {
            var chromeRegistry = Components.classes["@mozilla.org/chrome/chrome-registry;1"].getService(Components.interfaces.nsIXULChromeRegistry);
            var defaultLocale = chromeRegistry.getSelectedLocale("global");
            localeToSet = defaultLocale.split('-')[0];
            countryToSet = defaultLocale.split('-')[1];
        }
        var changeLang = Foxlingo.Utils.changeLanguage(localeToSet, "long");        
        Foxlingo.Prefs.setCharPref("clientfflocale", localeToSet + '-' + countryToSet);
        
        if (changeLang!="english") {
            pub.setLangToByLocale(countryToSet, "country");
            pub.setLangToByLocale(changeLang, "locale");
            if (Foxlingo.isAvailableLanguageFrom(changeLang)) {
                Foxlingo.Prefs.setCharPref("translatedsearchlangfrom", changeLang);
                Foxlingo.Prefs.setBoolPref("firsttimetranslatedsearch", false);        
            }
            Foxlingo.Prefs.setCharPref("options-dlang", changeLang);
            Foxlingo.Prefs.setCharPref("tempdlang", changeLang);
        }
        
        Foxlingo.Prefs.setCharPref("options-currency", "eur"); 
        Foxlingo.Prefs.setCharPref("tempcurrency", "eur");
        Foxlingo.Utils.setCurrencyAccordingToCountry();
        Foxlingo.Prefs.setBoolPref("changedlangto", false);

        if (!Foxlingo.Prefs.getBoolPref("isShowInstaller") && !document.getElementById("foxlingo-install-dialog")) {
            Foxlingo.Prefs.setBoolPref("usetoolbar", true);
            return;
        }
                
        initializeCommonOptions();

        for (var language in Foxlingo.Languages) {
            if (Foxlingo.isAvailableLanguageFrom(language))
                Foxlingo.session[language + "-toolbartemp"] = Foxlingo.Prefs.getBoolPref(language + "-toolbar", true);
            if (Foxlingo.isAvailableLanguageTo(language)) {
                if (language==Foxlingo.Utils.changeLanguage(localeToSet, "long") || language=="english")
                    Foxlingo.Prefs.setBoolPref(language + "-langto", true); 
            }
        }
        
        pub.setSearchCheckboxesLabels(true);
        document.getElementById("engflagQI").selectedIndex = (Foxlingo.Prefs.getCharPref("engflag", true)=="usflag" ? 0 : 1);
        document.getElementById("freflagQI").selectedIndex = (Foxlingo.Prefs.getCharPref("freflag", true)=="frflag" ? 0 : 1);
        document.getElementById("spaflagQI").selectedIndex = (Foxlingo.Prefs.getCharPref("spaflag", true)=="spflag" ? 0 : 1);
        document.getElementById("usestatusbarQI").checked = Foxlingo.Prefs.getBoolPref("usestatusbar", true);
        //document.getElementById("searchHomePageQI").checked = Foxlingo.Prefs.getBoolPref("searchHomePage", true);
        //document.getElementById("searchFirefoxSearchQI").checked = Foxlingo.Prefs.getBoolPref("searchFirefoxSearch", true);
    };
            
    pub.initializeOptions = function() {
        initializeCommonOptions();
        
        for (var language in Foxlingo.Languages) { 
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;
            
            var checkValue;
            if (Foxlingo.Prefs.getCharPref("firsttimetoolbarflags", true)=="true") {
                var langFromElement = document.getElementById(language + '-langfrom');
                checkValue = langFromElement.checked;
            } else {
                checkValue = Foxlingo.Prefs.getBoolPref(language + "-toolbar", true);
            }
            Foxlingo.session[language + "-toolbartemp"] = checkValue;
        }
        
        document.getElementById('showbuttontext').disabled = (Foxlingo.Prefs.getCharPref("customizeshowbuttontext", true)=="text");
    };
    
    pub.saveOptions = function() {   
        var i;
        
        Foxlingo.Options.ToolbarFlags.saveToolbarFlags('saveoptions');
        Foxlingo.Prefs.setCharPref("iconsizefl", Foxlingo.Toolbar.toolattr.getAttribute("iconsizefl"));
        
        for (var language in Foxlingo.Languages) {
            if (Foxlingo.isAvailableLanguageTo(language))
                pub.saveOption(language + "-langto");
            if (Foxlingo.isAvailableLanguageFrom(language))
                pub.saveOption(language + "-langfrom");
        }
        
        for each (var translator in Foxlingo.Translation.Translators)
            pub.saveOption(translator);
            
        Foxlingo.Prefs.setCharPref("options-dlang", Foxlingo.Prefs.getCharPref("tempdlang")); 
        Foxlingo.Prefs.setCharPref("options-currency", Foxlingo.Prefs.getCharPref("tempcurrency"));
        if (Foxlingo.Prefs.getCharPref("tempdlang")!="english" && !Foxlingo.Prefs.getBoolPref("changedtsearchlangfrom"))
            Foxlingo.Search.Translated.updateTranslatedSearchLangTo("english");
        
        for each (var boolOption in pub.BoolOptions)
            pub.saveOption(boolOption);
        
        for (var boolRadioOptionsParent in pub.BoolRadioOptionsParents) {
            for each (var boolRadioOption in pub.BoolRadioOptionsParents[boolRadioOptionsParent]) {
                if (document.getElementById(boolRadioOption).getAttribute("selected")=="true")
                    pub.saveOption(boolRadioOption, boolRadioOptionsParent, true);
            }
        }
        
        for each (var toolbarButton in Foxlingo.Toolbar.ToolbarButtons) { 
            var prefValue = (document.getElementById('button-' + toolbarButton).getAttribute("checked")=="true");
            Foxlingo.Prefs.setBoolPref('button-' + toolbarButton, prefValue); 
        }
        
        Foxlingo.Toolbar.Flags.removeAllChildrenFromToolbarFlags();
        Foxlingo.Toolbar.updateToolbarButtons();
        Foxlingo.Toolbar.updateToolbarButtonsText();
        Foxlingo.Toolbar.Flags.populateToolbarFlags();

        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var enumerator = windowMediator.getEnumerator("navigator:browser");
        var browserWindow = windowMediator.getMostRecentWindow("navigator:browser");
        var mainDocument = browserWindow.document;
        while(enumerator.hasMoreElements()) {
            var win = enumerator.getNext();
            win.setTimeout("Foxlingo.Options.updateOptions();", 3000);
            if (!Foxlingo.Prefs.getBoolPref("changedtsearchmanual"))
                win.setTimeout("Foxlingo.Options.updateTSearchManual();", 3000);
            if (!win.document.getElementById("Foxlingo_mainmenu-item"))
                continue;
            if  (document.getElementById("fulllogo") && document.getElementById("fulllogo").getAttribute("checked")=="true")
                win.document.getElementById("Foxlingo_mainmenu-item").setAttribute("image", "chrome://foxlingo/skin/foxlingo59x16.png");
            else
                win.document.getElementById("Foxlingo_mainmenu-item").setAttribute("image", "chrome://foxlingo/skin/foxlingo16x16.png");
        }
       
        Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).savePrefFile(null);
    };
    
    pub.updateTSearchManual = function() {
        Foxlingo.Search.Translated.updateTranslatedSearchLangTo("english");
        Foxlingo.Search.Translated.updateTranslatedSearchLangFrom(Foxlingo.Prefs.getCharPref("options-dlang"), false);
    };

    pub.updateOptions = function() {
        Foxlingo.Menus.createAllMenus(true);
        Foxlingo.Toolbar.resize();
//        cycleTabs();
        Foxlingo.Menus.buildFlagsOverflowMenu();
        Foxlingo.Search.Dictionary.updatePromtIcon();
    };
    
    pub.cancelOptions = function() {
        if (Foxlingo.Prefs.getCharPref('firsttimetoolbarflags')=='falsetemp')
            Foxlingo.Prefs.setCharPref('firsttimetoolbarflags', 'true');
            
        Foxlingo.Toolbar.toolattr.setAttribute("fulllogo", Foxlingo.Prefs.getBoolPref("fulllogo"));
        Foxlingo.Prefs.setCharPref("tempdlang", Foxlingo.Prefs.getCharPref("options-dlang"));
        Foxlingo.Prefs.setCharPref("tempcurrency", Foxlingo.Prefs.getCharPref("options-currency")); 
        Foxlingo.Toolbar.Flags.cancelToolbarFlags();
//        window.opener.setTimeout(function() { Foxlingo.Toolbar.Flags.populateToolbarFlags(); }, 200);
        Foxlingo.Toolbar.Flags.populateToolbarFlags();
        Foxlingo.Toolbar.updateToolbarButtons();
        Foxlingo.Toolbar.updateToolbarButtonsText();
    };
    
    pub.updateInstantTranslationOption = function(listValue) {
        var instantItem = document.getElementById("autotranslate");
        
        if (!listValue)
            return;
            
        if (listValue=="malay") {
            instantItem.setAttribute("disabled", true);
            instantItem.setAttribute("checked", false);
        } else {
            instantItem.setAttribute("disabled", false);
        }
    };
        
    pub.toggleTranslators = function() {
        pub.toggleAll('translators');
        pub.changedMenuOptions = true;
    };

    pub.onLangOptionClick = function(language) {
        Foxlingo.session[language + "-toolbartemp"] = !this.checked;
        Foxlingo.Toolbar.Flags.populateToolbarFlags(true);
        pub.changedMenuOptions = true;
    };

    pub.onOptionCommand = function(type) {
        pub.updateToggleAllButton(type);
        pub.changedMenuOptions = true;
    };

    pub.onGoogleTranslatorCommand = function() {
        pub.updateToggleAllButton('translators');
        pub.changedMenuOptions = true;
    };

    pub.updateChangedLangTo = function() {
        if (Foxlingo.Prefs.getBoolPref("changedlangto"))
            return;

        Foxlingo.Prefs.setBoolPref('changedlangto', true);
        var tempItem = document.getElementById("onetranslatorperpair");
        if (tempItem) 
            tempItem.checked = false;
    };
    
    pub.initializeOption = function(option, parent) {
        if (!document.getElementById(option))
            return;
        if (!parent) { 
            document.getElementById(option).checked = Foxlingo.Prefs.getBoolPref(option, true);
        } else {
            var bool = (Foxlingo.Prefs.getCharPref(parent, true)==option);
            document.getElementById(option).setAttribute("selected", bool);
        }
    };
    
    pub.saveOption = function(option, parent, attribute) {
        if (!parent) {
            if (document.getElementById(option))
                Foxlingo.Prefs.setBoolPref(option, document.getElementById(option).checked);
        } else {
            Foxlingo.Prefs.setCharPref(parent, option);
            if (attribute)
                Foxlingo.Toolbar.toolattr.setAttribute(parent, option);
        }
    };
    
    pub.setLangToByLocale = function(localeCode, type) {
        if (type=="country") {
            for (var lang in Foxlingo.Utils.domains) {
                if (lang!="english" && Foxlingo.Utils.domains[lang].indexOf(localeCode.toLowerCase())!=-1)
                    pub.setLangToOption(lang, true);
            }
        } else {
            pub.setLangToOption(localeCode, true);
        }
        switch (localeCode.toLowerCase()) {
                case "us" : 
                case "uk" :
                case "ca" : 
                    pub.setLangToOption("french", true);
                    break;
                case "be" : 
                    pub.setLangToOption("dutch", true);
                    pub.setLangToOption("french", true);
                    break;
                case "japanese" :
                case "chineses" :
                case "chineset" :
                case "korean" : 
                    pub.setLangToOption("korean", true);
                    pub.setLangToOption("japanese", true);
                    pub.setLangToOption("chineses", true);
                    pub.setLangToOption("chineset", true);
                    break;
                case "portuguese" :
                case "portugueseb" :
                    pub.setLangToOption("portuguese", true);
                    pub.setLangToOption("portugueseb", true);
                    break;            
                case "galician" :
                case "catalan" :
                case "aranese" :
                case "spanishla" :
                case "spanish" :
                    pub.setLangToOption("spanishla", true);
                    pub.setLangToOption("spanish", true);
                    pub.setLangToOption("galician", true);
                    pub.setLangToOption("catalan", true);
                    pub.setLangToOption("aranese", true);
                    break;
                case "il" :
                case "hebrew" :
                    pub.setLangToOption("arabic", true);
                    pub.setLangToOption("russian", true);
                    pub.setLangToOption("hebrew", true);
                    break;
                case "ukrainian" : 
                    pub.setLangToOption("russian", true);
                    break;
                default: break;
        }
    };

    pub.setLangToOption = function(lang) {
        for (var language in Foxlingo.Languages) {
            if (Foxlingo.isAvailableLanguageTo(language)) {
                Foxlingo.Prefs.setBoolPref(lang + '-langto', true);
                var tempOption = document.getElementById(lang + '-langto');
                if (tempOption)
                    tempOption.checked = valToSet;
            }
        }
    };
    
    pub.setSearchCheckboxesLabels = function(isQuickInstall) {
        //var quickInstallLabel = "";
        //if (isQuickInstall)
        //    quickInstallLabel = "QI";
        //var strbundle = document.getElementById("foxlingo-stringbundle");
        //var providerName = "Ask.com";
        //if (Foxlingo.Prefs.getCharPref("defaultsearchengine", true)=="zugoBing")
        //    providerName = "Bing.com";
        //if (Foxlingo.Prefs.getCharPref("defaultsearchengine", true)=="zugoYahoo")
        //    providerName = "Yahoo.com";
        //var searchHomePageCheckbox = document.getElementById("searchHomePage" + quickInstallLabel);
        //var stringBundle = document.getElementById("foxlingo-stringbundle");
        //if (searchHomePageCheckbox)
        //    searchHomePageCheckbox.label = stringBundle.getString("foxlingo.searchHomePage").replace("%S", providerName);
        //var searchFirefoxSearchCheckbox = document.getElementById("searchFirefoxSearch" + quickInstallLabel);
        //if (searchFirefoxSearchCheckbox)
        //    searchFirefoxSearchCheckbox.label = stringBundle.getString("foxlingo.searchFirefoxSearch").replace("%S", providerName);
        //var searchGroupBox = document.getElementById("searchGroupBox" + quickInstallLabel);
        //var isFromMozilla = Foxlingo.Utils.getIsFromMozilla();
        //var isFromUs = (Foxlingo.Prefs.getCharPref("foundipcountry").toLowerCase()=="us");
        //Foxlingo.Utils.debug("isFromMozilla: " + isFromMozilla + " isFromUs: " + isFromUs);
        //if (!isFromMozilla) {
        //    if (searchHomePageCheckbox) {
        //        Foxlingo.Utils.debug("Setting: searchHomePageCheckbox (in) " + searchHomePageCheckbox + " true");
        //        searchHomePageCheckbox.checked = true;
        //        Foxlingo.Prefs.setBoolPref("searchHomePage", true);
        //    }
        //    Foxlingo.Utils.debug("Setting: searchFirefoxSearchCheckbox " + searchFirefoxSearchCheckbox + " true");
        //    if (searchFirefoxSearchCheckbox) {
        //        searchFirefoxSearchCheckbox.checked = true;
        //        Foxlingo.Prefs.setBoolPref("searchFirefoxSearch", true);
        //    }
        //} else {
        //    Foxlingo.Utils.debug("Setting: searchHomePageCheckbox " + searchHomePageCheckbox + " false");
        //    if (searchHomePageCheckbox) {
        //        searchHomePageCheckbox.checked = false;
        //        Foxlingo.Prefs.setBoolPref("searchHomePage", false);
        //    }
        //    Foxlingo.Utils.debug("Setting: searchFirefoxSearchCheckbox " + searchFirefoxSearchCheckbox + " false");
        //    if (searchFirefoxSearchCheckbox) {
        //        searchFirefoxSearchCheckbox.checked = false;
        //        Foxlingo.Prefs.setBoolPref("searchFirefoxSearch", false);
        //    }
        //}
        //if (!isFromUs && isFromMozilla) {
        //    Foxlingo.Utils.debug("Setting: searchGroupBox " + searchGroupBox + " true");
        //    if (searchGroupBox)
        //        searchGroupBox.setAttribute("collapsed","false"); //stylish requires always false
        //    Foxlingo.Utils.debug("Setting: searchHomePageCheckbox " + searchHomePageCheckbox + " false");
        //    if (searchHomePageCheckbox) {
        //        searchHomePageCheckbox.checked= false;
        //        Foxlingo.Prefs.setBoolPref("searchHomePage", false);
        //    }
        //    Foxlingo.Utils.debug("Setting: searchFirefoxSearchCheckbox " + searchFirefoxSearchCheckbox + " false");
        //    if (searchFirefoxSearchCheckbox){
        //        searchFirefoxSearchCheckbox.checked= false;
        //        Foxlingo.Prefs.setBoolPref("searchFirefoxSearch", false);
        //    }
        //} else {
        //    Foxlingo.Utils.debug("Setting: searchGroupBox " + searchGroupBox + " false");
        //    if (searchGroupBox)
        //        searchGroupBox.setAttribute("collapsed","false");
        //}
    };

    //private functions
    function createTranslatorOptionsElements(type) {
        var strbundle = document.getElementById("foxlingo-stringbundle");
        var elements = [];

        for each (var translator in Foxlingo.Translation.Translators) {
            var tempItem = document.createElement("checkbox");
            var label = strbundle.getString("foxlingo.translator." + translator);
            tempItem.setAttribute("id", translator);
            tempItem.setAttribute("style", "margin-bottom: 0px;");
            tempItem.setAttribute("src", "chrome://foxlingo/skin/translators/" + translator + ".png");
            tempItem.setAttribute("oncommand", "Foxlingo.Options.onOptionCommand('translators');");
            if (translator=='google')
                tempItem.setAttribute("oncommand", "Foxlingo.Options.onGoogleTranslatorCommand();");
            tempItem.setAttribute("label", label);
            tempItem.setAttribute("class", "menuitem-iconic");
            elements.push(tempItem);
        }
        return elements;
    }

    function createLanguageOptionsElements(type) {
        var strbundle = document.getElementById("foxlingo-stringbundle");
        var elements = [];

        for (var language in Foxlingo.Languages) {
            if ((type=="langto" && !Foxlingo.isAvailableLanguageTo(language)) ||
                (type=="langfrom" && !Foxlingo.isAvailableLanguageFrom(language)))
                continue;
            var tempItem = document.createElement("checkbox");
            var label = strbundle.getString("foxlingo.language." + language);
            tempItem.setAttribute("id", language + "-" + type);
            tempItem.setAttribute("style", "margin-bottom: -3px;");
            
            if (type=="langto") 
                tempItem.setAttribute("onclick", "Foxlingo.Options.updateChangedLangTo(); Foxlingo.changedMenuOptions = true;");
            else if (type=="langfrom" && Foxlingo.Prefs.getCharPref('firsttimetoolbarflags', true)!='falsefinal')
                tempItem.setAttribute("onclick", "Foxlingo.Options.onLangOptionClick('" + language + "');");
            tempItem.setAttribute("oncommand", "Foxlingo.Options.onOptionCommand('" + type + "');");
            var src = "chrome://foxlingo/skin/flags-s/" + language + Foxlingo.Utils.getSecondFlag(language);
            tempItem.setAttribute("src", src);
            tempItem.setAttribute("label", label);
            tempItem.setAttribute("class", "menuitem-iconic");
            elements.push(tempItem);
        }
        return elements;
    }

    function createOptions(type, columnlength) {
        var strbundle = document.getElementById("foxlingo-stringbundle");
        var columns = document.getElementById("options-" + type);

        if (type=="dlang") {
            Foxlingo.Menus.createDlangMenuItem(columns);
            return;
        }

        if (type=="currency") {
            Foxlingo.Menus.createCurrencyMenuItem(columns);
            return;
        }
        Foxlingo.Utils.removeChildren(columns);
        
        var col = -1;
        var row = 0;
        var elements;

        if (type=="translators")
            elements = createTranslatorOptionsElements();
        else //(type=="langto" || type=="langfrom")
            elements = createLanguageOptionsElements(type);

        var tempItem2;

        for each (var element in elements) {
            if (row%columnlength===0) { //new column
                col++;

                var tempItem = document.createElement("column");
                tempItem.setAttribute("id", "options-" + type + col);
                columns.appendChild(tempItem);
                
                var menu = document.getElementById("options-" + type + col);
                Foxlingo.Utils.removeChildren(menu);

                if (col===0) { //first column
                    //create toggle all button
                    tempItem2 = document.createElement("checkbox");
                    var label = strbundle.getString("foxlingo.text.toggle.all");
                    tempItem2.setAttribute("id", "toggleall-" + type);
                    tempItem2.setAttribute("label", label);
                    if (type=="langto")
                        tempItem2.setAttribute("oncommand", "Foxlingo.Options.toggleAll('langto'); Foxlingo.changedMenuOptions = true;");
                    else if (type=="langfrom")
                        tempItem2.setAttribute("oncommand", "Foxlingo.Options.toggleAll('langfrom'); Foxlingo.changedMenuOptions = true;");
                    else if (type=="translators")
                        tempItem2.setAttribute("oncommand", "Foxlingo.Options.toggleTranslators()");
                    menu.appendChild(tempItem2);
                } else {
                    tempItem2 = document.createElement("spacer");
                    tempItem2.setAttribute("style", "min-width: 15px");
                    tempItem2.setAttribute("flex", "1");
                    menu.appendChild(tempItem2);
                }
                row++;
            }

            menu.appendChild(element);
            row++;
        }
    }   

    function initializeCommonOptions() {
        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var browserWindow = windowMediator.getMostRecentWindow("navigator:browser");
        var toolbarbutton = browserWindow.document.getElementById("foxlingo-toolbar-2-4-3c");
        var disableToolbarCheckbox = document.getElementById("disableToolbar");
        if (disableToolbarCheckbox) {
            if (toolbarbutton && toolbarbutton.collapsed)
                disableToolbarCheckbox.setAttribute("checked", "true");
            else
                disableToolbarCheckbox.setAttribute("checked", "false");
        }
	pub.setSearchCheckboxesLabels();
        createOptions('langto', 20);
        createOptions('langfrom', 20);
        createOptions('translators', 12);
        createOptions('dlang');
        createOptions('currency');
        
        for (var language in Foxlingo.Languages) {
            if (Foxlingo.isAvailableLanguageFrom(language))
                pub.initializeOption(language + "-langfrom");
            if (Foxlingo.isAvailableLanguageTo(language))
                pub.initializeOption(language + "-langto");
        }
        
        for each (var translator in Foxlingo.Translation.Translators)
            pub.initializeOption(translator);
            
        for each (var boolOption in pub.BoolOptions)
            pub.initializeOption(boolOption);
            
        for each (var toolbarButton in Foxlingo.Toolbar.ToolbarButtons)
            pub.initializeOption('button-' + toolbarButton);
            
        for (var boolRadioOptionsParent in pub.BoolRadioOptionsParents) {
            for each (var boolRadioOption in pub.BoolRadioOptionsParents[boolRadioOptionsParent])
                pub.initializeOption(boolRadioOption, boolRadioOptionsParent);
        }
            
        pub.updateToggleAllButton('langto');
        pub.updateToggleAllButton('langfrom');
        pub.updateToggleAllButton('translators');
        
        Foxlingo.Options.ToolbarFlags.setToolbarFlagsOption(!Foxlingo.Prefs.getBoolPref('toolbarflags', true));
    }
    
    function cycleTabs() {
        if (!document.getElementById("content"))
            return;
    }
    
    function isAllToggled(type, isChecked) {
        if (type=='translators') {
            for each (var translator in Foxlingo.Translation.Translators) {
                if (document.getElementById(translator).checked!=isChecked)
                    return false;
            }
            return true;
        }

        for (var language in Foxlingo.Languages) {
            var element = null;

            if (language=='langto' && Foxlingo.isAvailableLanguageTo(language)) {
                element = language + '-langto';
            } else if (Foxlingo.isAvailableLanguageFrom(language)) {
                if (type=='langfrom')
                    element = language + '-langfrom';
                else if (type=='toolbarflags')
                    element = 'toolbarflag-' + language;
            }

            if (element && document.getElementById(element).checked!=isChecked)
                return false;
        }
        return true;
    }

    return pub;
}();

//added Cancel and OK buttons on Mac and *nix platforms
function foxlingoPrefwindowButtons(){
	var platform = navigator.platform.toLowerCase();
	var isWindows = ((platform.indexOf("win") != -1) ? true : false);
	if(!isWindows){
		try{	
			var buttonAccept=document.documentElement.getButton('accept');
		    buttonAccept.hidden = false;
		    buttonAccept.disabled = false;
		}catch(e){;}

		try{
			document.getElementById("foxlingoAcceptBtn").hidden = false;
			document.getElementById("foxlingoAcceptBtn").disabled = false;
		}catch(e){;}
        try{
			document.getElementById("foxlingoAcceptBtn").hidden = false;
			document.getElementById("foxlingoCancelBtn").hidden = false;
		}catch(e){;}
	}
	return true;
}
