/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Search.Highlight = function() {
    var pub = {};

    //private properties
    //constants
    const FL_Finder = Components.classes['@mozilla.org/embedcomp/rangefind;1'].createInstance().QueryInterface(Components.interfaces.nsIFind);

    const HighlightColors = ["background: #FF0;", "background: #0FF;", "background: #0F0;", "background: #F0F;", "background: orange;", "background: dodgerblue;"];

    //private members
    var mLastHighlightedTerms = "";


    //public functions
    pub.toggleHighlighting = function() {
        if (!Foxlingo.Search.getSearchTerms())
            return;
            
        var hb = document.getElementById("Foxlingo_highlighter-item");
        
        if (!hb && window.opener)
            hb = window.opener.document.getElementById("Foxlingo_highlighter-item");
            
        if (!hb)
            return;

        hb.checked = !hb.checked;
        if (!hb.checked)
            removeHighlighting(null);
        else  
            addHighlighting(null);
        Foxlingo.Search.updateSearchWordButtons();
    };
    
    pub.checkHighlighting = function() {
        var hbutton = document.getElementById("Foxlingo_highlighter-item");

        if (!hbutton && window.opener)
            hbutton = window.opener.document.getElementById("Foxlingo_highlighter-item");

        if (!hbutton)
            return;

        var searchWordsContainer = document.getElementById("Foxlingo_searchwordbuttons-item");
        if (!searchWordsContainer && window.opener)
            searchWordsContainer = window.opener.document.getElementById("Foxlingo_searchwordbuttons-item");

        if (!searchWordsContainer)
            return;

        if (!hbutton || !searchWordsContainer)
            return;

        if (searchWordsContainer.childNodes.length > 1) {
            hbutton.setAttribute("disabled", false);
        } else {
            hbutton.checked = false;
            hbutton.setAttribute("disabled", true);
            removeHighlighting(null);
        }
    };
    

    //private functions
    function addHighlighting(win) {
        doHighlightingOperation(win, true);
    }

    function removeHighlighting(win) {
        doHighlightingOperation(win, false);
    }
    
    function doHighlightingOperation(win, isAdd) {
        var i;

        if (!win)
            win = window._content;
        if (!win && window.opener)
            win = window.opener._content;
        if (!win)
            return;

        if (win.frames) {
            for (i = 0; i< win.frames.length; i++) {
                var frame = win.frames[i];
                if (isAdd)
                    addHighlighting(frame);
                else
                    removeHighlighting(frame);
            }
        }

        var doc = win.document;
        if (!doc)
            return;

        var body = doc.body;
        if (!body)
            return;
        
        var terms;
        if (isAdd) {
            terms = Foxlingo.Utils.trimString(Foxlingo.Search.getSearchTerms());
            mLastHighlightedTerms = terms;
        } else {
            terms = mLastHighlightedTerms;
        }
        var termsArray = Foxlingo.Utils.splitTerms(terms);

        var count = body.childNodes.length;
        for (i in termsArray) {
            var term = termsArray[i];
            if (isAdd) {
                var span = doc.createElement("span");
                span.setAttribute("style", HighlightColors[i%HighlightColors.length] + " color: #000;");
                span.setAttribute("class", "highlighted");
            }

            var searchRange = doc.createRange();
            searchRange.setStart(body, 0);
            searchRange.setEnd(body, count);

            var start = doc.createRange();
            start.setStart(body, 0);
            start.setEnd(body, 0);

            var end = doc.createRange();
            end.setStart(body, count);
            end.setEnd(body, count);

            var result;
            var parent;
            while ((result = FL_Finder.Find(term, searchRange, start, end))) {
                var startContainer = result.startContainer;
                var docfrag;
                if (isAdd) {
                    var surround = span.cloneNode(true);
                    var startOffset = result.startOffset;
                    var endOffset = result.endOffset;
                    docfrag = result.extractContents();
                    var before = startContainer.splitText(startOffset);
                    parent = before.parentNode;
                    
                    surround.appendChild(docfrag);
                    parent.insertBefore(surround, before);
                    start = surround.ownerDocument.createRange();
                    start.setStart(surround, surround.childNodes.length);
                    start.setEnd(surround, surround.childNodes.length);
                } else {
                    var elem;
                    try {
                        elem = startContainer.parentNode;
                    } catch (e) {
                        Foxlingo.Utils.debugException(e);
                    }

                    var child = null;
                    var next = null;
                    parent = null;
                    while (elem) { 
                        if (elem instanceof HTMLDocument)
                            break;
                        try {
                            if (elem.getAttribute("class")=="highlighted") {
                                docfrag = doc.createDocumentFragment();
                                next = elem.nextSibling;
                                parent = elem.parentNode;
                                while (child = elem.firstChild)
                                    docfrag.appendChild(child);
                                
                                start = doc.createRange();
                                start.setStartAfter(elem);
                                parent.removeChild(elem);
                                parent.insertBefore(docfrag, next);
                                break;
                            }
                        } catch(e) {
                            Foxlingo.Utils.debugException(e);
                        }
                        elem = elem.parentNode;
                    }
                    start = doc.createRange();
                    start.setStart(result.endContainer, result.endOffset);
                    start.collapse(true);            
                }
            }
        }
    }


    return pub;
}();
