/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Search = function() {
    var pub = {};

    //public properties
    pub.tempSearchTerms = '';


    //private properties
    //constants
    const FL_Sound = Components.classes["@mozilla.org/sound;1"].createInstance(Components.interfaces.nsISound);

    const StylesArray = ["-moz-image-region: rect(0px 32px 16px 16px)", "-moz-image-region: rect(0px 48px 16px 32px)", "-moz-image-region: rect(0px 64px 16px 48px)",
                         "-moz-image-region: rect(0px 80px 16px 64px)", "-moz-image-region: rect(0px 96px 16px 80px)", "-moz-image-region: rect(0px 112px 16px 96px)"];

    //private members
    var mHistory = null;


    //public functions
    pub.init = function() {
        if (Foxlingo.Utils.isFF)
            mHistory = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2 || Components.interfaces.nsIFormHistory);
    };

    pub.combinedSearch = function(event) {
        var searchType;
        var defaultSearchEngine = Foxlingo.Prefs.getCharPref("defaultsearchengine")
            
        if (defaultSearchEngine=="zugoAsk" || defaultSearchEngine=="zugoBing" ||
            defaultSearchEngine=="zugoYahoo") 
            searchType = defaultSearchEngine;
        else
            searchType = "IxquickWeb";
            
        pub.prepareSearch(event, searchType);
    };
    
    pub.linkularSearch = function(event) {
        pub.prepareSearch(event, "LinkularSearch");
    };
       
    pub.updateImageCombinedSearch = function(event, type) {
        var combined = document.getElementById("Foxlingo_search-item");
        if (!combined)
            return;
        combined.setAttribute("searchType", type);
        pub.prepareSearch(event, type);
    };
    
    pub.updateSearchTerms = function() {
        var sBox = document.getElementById('Foxlingo_searchterms-item');
        if (sBox && pub.tempSearchTerms)
            sBox.value = pub.tempSearchTerms;
        pub.termsHaveUpdated();
        pub.tempSearchTerms = null;
    };
    
    pub.clearSearchHistory = function() {
        var stringBundle = document.getElementById("foxlingo-stringbundle");
        if (!confirm(stringBundle.getString("foxlingo.text.confirm.clearSearch")))
            return;
        Foxlingo.Prefs.setCharPref("searchhistory", "");
        var searchTermsItem = document.getElementById("Foxlingo_searchterms-popup");
        if (!searchTermsItem && window.opener)
            searchTermsItem = window.opener.document.getElementById("Foxlingo_searchterms-popup");
            
        if (searchTermsItem && searchTermsItem.hasChildNodes()) {
            while (searchTermsItem.childNodes.length>0) {
                searchTermsItem.removeChild(searchTermsItem.firstChild);
            }
        }

        var history = Components.classes["@mozilla.org/satchel/form-history;1"].getService(Components.interfaces.nsIFormHistory2 || Components.interfaces.nsIFormHistory);
        history.removeEntriesForName("Foxlingo-Search-History");
    };
    
    pub.termsHaveUpdated = function() {
        pub.updateSearchWordButtons();
        Foxlingo.Search.Highlight.checkHighlighting();
    };

    pub.findInPage = function(term, e) { 
        var focusedWindow = document.commandDispatcher.focusedWindow;
        
        if (!focusedWindow || focusedWindow==window)
            focusedWindow = window._content;
            
        var findInst = getBrowser().webBrowserFind;
        var findInFrames = findInst.QueryInterface(Components.interfaces.nsIWebBrowserFindInFrames);
        
        findInFrames.rootSearchFrame = window._content;
        findInFrames.currentSearchFrame = focusedWindow;
        findInst.searchString = term; 
        if (e) { 
            findInst.searchFrames = true;
            findInst.wrapFind = true;
            findInst.entireWord = e.altKey;
            findInst.findBackwards = e.shiftKey;
            findInst.matchCase = e.ctrlKey;
            if (findInst.entireWord)
                findInst.searchString = ' ' + str + ' ';
        }
        
        var found = findInst.findNext();
        var stringBundle = document.getElementById("foxlingo-stringbundle");
        var thisTerm = "\'" + term + "\'";
        
        if (!found) {
            FL_Sound.beep();
            window._content.status = stringBundle.getFormattedString("foxlingo.FL_TermNotFound", [thisTerm]);
        }
    };
    
    pub.updateSearchWordButtons = function() {
        pub.clearSearchWordButtons();
        addSearchWordButton(pub.getSearchTerms());
    };
    
    pub.clearSearchWordButtons = function() {
        var i, node;
        var searchWordsContainer = document.getElementById("Foxlingo_searchwordbuttons-item");

        if (!searchWordsContainer && window.opener)
            searchWordsContainer = window.opener.document.getElementById("Foxlingo_searchwordbuttons-item");

        if (!searchWordsContainer)
            return;

        for (i = searchWordsContainer.childNodes.length - 1; i >= 0; i--) {
            node = searchWordsContainer.childNodes[i];
            if (node && node.id!="Foxlingo_wordfindCustomize")
                searchWordsContainer.removeChild(node);
        }

        var overflowMenu = document.getElementById("Foxlingo_Overflow-Menu");
        if (!overflowMenu && window.opener)
            overflowMenu = window.opener.document.getElementById("Foxlingo_Overflow-Menu");

        if (!overflowMenu)
            return;

        for (i = overflowMenu.childNodes.length - 1; i >= 0; i--) {
            node = overflowMenu.childNodes[i];
            if (node && node.id!="Foxlingo_wordfindCustomize")
                overflowMenu.removeChild(node);
        }
    };
    
    pub.resetSearchBox = function(url, selection) {
        var searchBox = "";
        if (document)
            searchBox = document.getElementById("foxlingo-search-box");
        else if (window.document)
            searchBox = window.document.getElementById("foxlingo-search-box");
        if (!searchBox)
            return;
        if (searchBox.inputField && ((url && url.indexOf("http")==0) || selection)) {
            if (!selection) {
                var queryStringName = "q";
                if (url.indexOf("search.foxlingo")>1 || url.indexOf("search.linkextend")>1)
                    queryStringName = "w";
                else if (url.indexOf(".yahoo.")>1)
                    queryStringName = "p";
                else if (url.indexOf(".ixquick.")>1)
                    queryStringName = "query";
                else if (url.indexOf(".babylon.")>1)
                    queryStringName = "term";
                var searchStringRegExp = new RegExp(queryStringName + "=([^&]*)");
                var searchString = url.match(searchStringRegExp);
                if (searchString)
                    searchString = searchString[1];
                if (searchString) {
                    searchString = decodeURIComponent(searchString.replace(/\+/gi, " "));
                    searchString = searchString.replace(/%3A/gi, ":");
                    searchString = searchString.replace(/%2B/gi, "+");
                }
            } else {
                var searchString = selection;
            }
            if (searchString && !Foxlingo.isPBS)
                searchBox.value = searchString; 
        }
        pub.termsHaveUpdated();
    }

    pub.prepareSearch = function(event, searchType) {
        if (!searchType)
            searchType = getSearchType(event);

        var searchTerms = Foxlingo.Utils.trimString(pub.getSearchTerms());
        searchTerms = encodeURIComponent(searchTerms); 
        searchTerms = Foxlingo.Utils.convertTermsToURI(searchTerms);
        addToSearchHistory(searchTerms);

        var isEmpty = false;
        if (searchTerms.length===0)
            isEmpty = true;
        
        if (Foxlingo.Utils.isFF && !isEmpty && mHistory)
            mHistory.addEntry("Foxlingo-Search-History", decodeURIComponent(searchTerms));
        
        if (Foxlingo.Prefs.getBoolPref("translatedsearch") && searchTerms.replace(/ /g,''))
            Foxlingo.Search.Translated.translatedSearch(searchTerms, searchType);
        else
            pub.search(searchTerms, searchType, isEmpty);
    };
    
    pub.search = function(searchTerms, searchType, isEmpty, target, isTranslatedSearch) {
        var win = window._content.document;
        var originalTerms = searchTerms;
        var FFstate = (Foxlingo.Prefs.getBoolPref("familyfilter") ? "on" : "off");
        
        if (!isEmpty)
            searchTerms = Foxlingo.Utils.convertTermsToURI(searchTerms);
        searchTerms = searchTerms.replace(/%0D|%0A/gi, ' ');
        
        var ixquickLang = Foxlingo.session.ixquickDefaultLang;
        if (!ixquickLang)
            ixquickLang = "english";
            
        var url = getSearchUrl(searchType, FFstate, searchTerms, isEmpty, isTranslatedSearch);
        
        if (target && Foxlingo.Prefs.getCharPref("opens")=="openCurrentS")
            target.location.href = url; 
        else
            Foxlingo.Utils.openTabS(url);  
    };
    
    pub.getSearchTerms = function() { 
        if (document.getElementById("foxlingo-search-box"))
            return document.getElementById("foxlingo-search-box").value; 
        return "";
    };
    
    pub.loadSearchHistory = function() {
        var searchHistory = Foxlingo.Prefs.getCharPref("searchhistory");
        if (searchHistory.length===0 || !document.getElementById("Foxlingo_searchterms-popup"))
            return;
        var tempButton = "";
        var searchTermsItem = document.getElementById("Foxlingo_searchterms-popup");
        searchHistory = searchHistory.split(";");
        for (var i = searchHistory.length - 1; i >= 0; i--) {
            if (searchHistory[i].replace(/ /g,'')!=='') {
                var tempButton = document.createElement("menuitem");
                var term = decodeURIComponent(searchHistory[i]).replace(/\+/gi, ' ');
                tempButton.setAttribute("label", term);
                tempButton.setAttribute("oncommand", "document.getElementById('foxlingo-search-box').value =  '" + term + "'; Foxlingo.Search.combinedSearch();");
                searchTermsItem.appendChild(tempButton);
            }
        }
    };
    

    //private functions
    function addSearchWordButton(term) {
        var searchWordsContainer = document.getElementById("Foxlingo_searchwordbuttons-item");
        var highlighter = document.getElementById("Foxlingo_highlighter-item");
        var overflowButton = document.getElementById("Foxlingo_Overflow-Button");
        var overflowMenu = document.getElementById("Foxlingo_Overflow-Menu");
        var stringBundle = document.getElementById("foxlingo-stringbundle");
        var searchTerms = Foxlingo.Utils.splitTerms(term);
        var tempButton;
        var tempMenuItem;
        
        if (!searchWordsContainer || !highlighter || !overflowButton || !overflowMenu)
            return;
        
        for (var i in searchTerms) {
            if (i>=20)
                break;
            var searchTerm = searchTerms[i];
            if (searchTerm.search(/^(?:site|cache|link|related|info|filetype|safesearch|allintitle|intitle|allinurl|inurl):/)!=-1 ||
                searchTerm.search(/^OR$/)!=-1 || searchTerm.search(/^-/)!=-1)
                continue; // Ignore this term 
            var safeTerm = searchTerm.replace(new RegExp("'", 'g'), "\\'"); 
            safeTerm = "\'" + safeTerm + "\'";

            tempButton = document.createElement("toolbarbutton");
            tempButton.setAttribute("label", searchTerm);
            tempButton.setAttribute("collapsed", "false");
            tempButton.setAttribute("foxlingo-show-text", "always");
            tempButton.setAttribute("foxlingo-show-icon", "always");
            tempButton.setAttribute("tooltiptext", stringBundle.getFormattedString("foxlingo.FindNextOccurrence", [searchTerm]));
            tempButton.setAttribute("oncommand", "Foxlingo.Search.findInPage(" + safeTerm + ", event)");
            tempButton.className = "SearchWordButton";
            if (!searchTermAppearing(searchTerm, searchWordsContainer))
                searchWordsContainer.appendChild(tempButton);

            tempMenuItem = document.createElement("menuitem");
            tempMenuItem.setAttribute("label",  searchTerm);
            tempMenuItem.setAttribute("class",  "foxlingo-overflow-item menuitem-iconic");
            tempMenuItem.setAttribute("collapsed", "true");
            tempMenuItem.setAttribute("foxlingo-show-text", "always");
            tempMenuItem.setAttribute("tooltiptext", stringBundle.getFormattedString("foxlingo.FindNextOccurrence", [searchTerm]));
            tempMenuItem.setAttribute("oncommand", "Foxlingo.Search.findInPage(" + safeTerm + ", event)");
            if (highlighter.checked) {            
                tempButton.setAttribute("style", StylesArray[i%StylesArray.length] + " !important;");
                tempMenuItem.setAttribute("style", StylesArray[i%StylesArray.length] + " !important;");
            }
            if (!searchTermAppearing(term, overflowMenu))
                overflowMenu.appendChild(tempMenuItem);
        }
        Foxlingo.Toolbar.resize();
    }
    
    function searchTermAppearing(searchTerm, searchTerms) {
        for (var i = 0; i<searchTerms.childNodes.length; i++) {
            if (searchTerms.childNodes[i].getAttribute("label")==searchTerm)
                return true;
        }
        return false;
    }
        
    function getSearchType(event) {
        var searchItem = document.getElementById("Foxlingo_search-item");
        if (searchItem)
            return searchItem.getAttribute("searchType");
        return "IxquickWeb"; 
    }
    
    function getSearchUrl(searchType, FFstate, searchTerms, isEmpty, isTranslatedSearch) {
        var url;

        if (!searchType) {
            searchType = "IxquickWeb";
            var defaultSearchEngine = Foxlingo.Prefs.getCharPref("defaultsearchengine")
            if (defaultSearchEngine=="zugoAsk" || defaultSearchEngine=="zugoBing"||
                defaultSearchEngine=="zugoYahoo") 
                searchType = defaultSearchEngine;
        }
            
        if (searchType=="IxquickWeb") {
            var ixquickLang = Foxlingo.session.ixquickDefaultLang;
            if (!ixquickLang)
                ixquickLang = "english";
            url = "http://www.ixquick.com/do/ptn?rl=FLN&ff=" + FFstate + "&language=" + ixquickLang + "&dest=";
            if (isEmpty)
                url = url + "/&language=" + ixquickLang;
            else
                url = url + "metasearch.pl&query=" +  searchTerms + "&cat=web";
        } else if (searchType=="zugoAsk") {
            url = "http://www.foxlingostart.com/s/?q=" +  searchTerms + "&site=Ask&pid=178&src=FF-Toolbar";
        } else if (searchType=="zugoBing") {
            url = "http://www.foxlingostart.com/s/?q=" +  searchTerms + "&site=Bing&pid=178&src=FF-Toolbar";
        } else if (searchType=="zugoYahoo") {
            url = "http://www.foxlingo-start.com/s/?q=" +  searchTerms + "&site=Yahoo&pid=178&src=FF-Toolbar";
        } else if (searchType=="ixquickImage") {
            if (isEmpty)
                url = "http://www.ixquick.com/do/ptn?rl=FLN&dest=/&language=english&cat=pics&ff=" + FFstate;
            else
                url = "http://ixquick.com/do/ptn?rl=FLN&dest=metasearch.pl&ff=" + FFstate +"&query=" + searchTerms + "&cat=pics";
        } else if (searchType=="LinkularSearch") {
            url = "http://www.linkular.com";
            if (!isEmpty)
                url += "/search/?q=" + searchTerms;
        } else {
            var language;
            switch (searchType) { 
                case "IxquickGeoUS": language = "english"; break;
                case "IxquickGeoUK": language = "english_uk"; break;
                case "IxquickGeoChineses": language = "jiantijongwen"; break;
                case "IxquickGeoChineset": language = "fantizhengwen"; break;
                case "IxquickGeoDanish": language = "dansk"; break;
                case "IxquickGeoDutch": language = "nederlands"; break;
                case "IxquickGeoFinnish": language = "suomi"; break;
                case "IxquickGeoFrench": language = "francais"; break;
                case "IxquickGeoGerman": language = "deutsch"; break;
                case "IxquickGeoItalian": language = "italiano"; break;
                case "IxquickGeoJapanese": language = "nihongo"; break;
                case "IxquickGeoKorean": language = "hangul"; break;
                case "IxquickGeoNorwegian": language = "norsk"; break;
                case "IxquickGeoPolish": language = "polski"; break;
                case "IxquickGeoPortuguese": language = "portugues"; break;
                case "IxquickGeoSpanish": language = "espanol"; break;
                case "IxquickGeoSwedish": language = "svenska"; break;
                case "IxquickGeoTurkish": language = "turkce"; break;
                default: language = ixquickLang; break;
            }
            if (isEmpty)
                url = "http://www.ixquick.com/do/ptn?rl=FLN&dest=/&language=" + language + "&ff=" + FFstate;
        }
        if (isTranslatedSearch) {
            var langto = Foxlingo.Prefs.getCharPref("translatedsearchlangfrom");
            var langfrom = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
            var translator = Foxlingo.Translation.getTranslator(langfrom, langto);
            langfrom = Foxlingo.Translation.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(langfrom, "shortName"), translator);
            langto = Foxlingo.Translation.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(langto, "shortName"), translator);
            url = Foxlingo.Translation.getTranslationUrl(translator, langfrom, langto, url);
        }
        return url;
    }
    
    function addToSearchHistory(historyItem) {
        if (!historyItem || !document.getElementById("Foxlingo_searchterms-popup") || Foxlingo.isPBS)
            return;

        var searchTermsItem = document.getElementById("Foxlingo_searchterms-popup");
        var searchTermPos = historyItemAppearing(decodeURIComponent(historyItem), searchTermsItem);
        
        if (searchTermPos)
            return;

                var tempButton = document.createElement("menuitem");
                var term = decodeURIComponent(historyItem);
                tempButton.setAttribute("label", term);
                tempButton.setAttribute("oncommand", "document.getElementById('foxlingo-search-box').value =  '" + term + "'; Foxlingo.Search.combinedSearch();");
                searchTermsItem.insertBefore(tempButton, searchTermsItem.firstChild);
            
        var searchHistory = Foxlingo.Prefs.getCharPref("searchhistory");
            
        if (!searchHistory)
            Foxlingo.Prefs.setCharPref("searchhistory", historyItem);
        else
            Foxlingo.Prefs.setCharPref("searchhistory", searchHistory + ';' + historyItem);
    }
    
    function historyItemAppearing(searchTerm, searchTerms) {
        var element = searchTerms;
        if (searchTerms.menupopup)
            element = searchTerms.menupopup;
            
        for (var i = 0; i<element.childNodes.length; i++) {
            if (element.childNodes[i].getAttribute("label")==searchTerm)
                return (i + 1);
        }
        return false;
    }
    

    return pub;
}();
