/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Search.Translated = function() {
    var pub = {};

    //private properties
    var mXmlHttp = new XMLHttpRequest();
    var mIsXmlHttpBusy = false;


    //public functions
    pub.updateTranslatedSearchLangFrom = function(lang, isManual) {
        var langFromMenu = document.getElementById("Foxlingo_translatedsearchmenu-langfrom");
        if (!langFromMenu)
            return;
        if (!Foxlingo.isAvailableLanguageFrom(lang))
            return;
        var dlang = Foxlingo.Prefs.getCharPref("options-dlang");
        var setChangeLangTo = false;
        var langTo;
        
        if (isManual) {
            if (!Foxlingo.Prefs.getBoolPref("changedtsearchmanual") && Foxlingo.Prefs.getCharPref("translatedsearchlangfrom")!="english")
                pub.updateTranslatedSearchLangTo("english", false);
            Foxlingo.Prefs.setBoolPref("changedtsearchlangfrom", true);
            Foxlingo.Prefs.setBoolPref("changedtsearchmanual", true);
        }
        
        var tempLangTo = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
        
        Foxlingo.Prefs.setCharPref("translatedsearchlangfrom", lang);
        langFromMenu.setAttribute("image", "chrome://foxlingo/skin/flags-s/" + lang + ".png");
        
        var tempItem;
        var tempTranslator;
        for (var language in Foxlingo.Languages) {
            tempItem = document.getElementById("tsearch-langfrom-" + language);
            if (!tempItem)
                continue;
            tempTranslator = Foxlingo.Translation.getTranslator(lang, language);
            if (tempTranslator=='none' || lang==language) {
                if (language==tempLangTo) {
                    if (Foxlingo.Translation.getTranslator(lang, "english")!='none')
                        langTo = "english";
                    else if (Foxlingo.Translation.getTranslator(lang, dlang)!='none')
                        langTo = dlang;
                    else if (Foxlingo.Translation.getTranslator(lang, "spanish")!='none')
                        langTo = "spanish";
                    pub.updateTranslatedSearchLangTo(langTo, false);
                    setChangeLangTo = true;
                }
                tempItem.collapsed = true;
                continue;
            }            
            if (!setChangeLangTo) {
                langTo = language;
                setChangeLangTo = true;
            }
            tempItem.collapsed = false;
        }
    };
    
    pub.toggleTranslatedSearch = function() {
        var toolbarsearchitem = document.getElementById("Foxlingo_translatedsearch-item");
        var toolbarsearchbuttonitem = document.getElementById("Foxlingo_search-item");
        var tempPref = Foxlingo.Prefs.getBoolPref("translatedsearch");
        var tooltip = document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.translated.search");
        if (tempPref) {
            toolbarsearchbuttonitem.setAttribute("image", "chrome://foxlingo/skin/search.png");
            toolbarsearchitem.setAttribute("image", "chrome://foxlingo/skin/translatedsearchoff.png");
            toolbarsearchitem.setAttribute("tooltiptext", tooltip + ' - ' + document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.off"));
        } else {
            toolbarsearchbuttonitem.setAttribute("image", "chrome://foxlingo/skin/search-translated.png");
            toolbarsearchitem.setAttribute("image", "chrome://foxlingo/skin/translatedsearch.png");
            toolbarsearchitem.setAttribute("tooltiptext", tooltip + ' - ' + document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.on"));
        }
        Foxlingo.Prefs.setBoolPref("translatedsearch", !tempPref); 
    };
    
    pub.updateTranslatedSearchLangTo = function(lang, isManual) {
        var langToMenu = document.getElementById("Foxlingo_translatedsearchmenu-langto");
        if (!langToMenu)
            return;
            
        if (isManual) 
            Foxlingo.Prefs.setBoolPref("changedtsearchmanual", true);
        
        if (lang)
            Foxlingo.Prefs.setCharPref("translatedsearchlangto", lang);
        langToMenu.setAttribute("image", "chrome://foxlingo/skin/flags-s/" + lang + ".png");
    };
    
    pub.translatedSearch = function(searchTerms, searchType) {
        var langfrom = Foxlingo.Prefs.getCharPref("translatedsearchlangfrom");
        var langto = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
        var translator = Foxlingo.Translation.getTranslator(langfrom, langto);
        var strbundle = document.getElementById("foxlingo-stringbundle");
        
        Foxlingo.session.translatedSearchTerms = searchTerms;
        Foxlingo.session.translatedSearchType = searchType;
        
        if (searchTerms.replace(/ /g,'')==='')
            return;
        searchTerms = Foxlingo.Utils.convertTermsToURI(searchTerms);
        Foxlingo.Status.showStatusMessage(strbundle.getString("foxlingo.text.translated.search.wait"));
        Foxlingo.Translation.Text.translateText(langfrom, langto, translator, "yes");
    };
    
    pub.translatedText = function(injectionProperties) {
        var langfrom = "english";
        var langto = Foxlingo.Prefs.getCharPref("options-dlang");
        var translator = Foxlingo.Translation.getTranslator(langfrom, langto);
        Foxlingo.Translation.Text.translateText(langfrom, langto, translator, "yes",null,injectionProperties);
    };
    
    pub.translatedSearchMalay = function(txt, translator, serviceName, lang1, lang2) {
        var checkUrl = "http://apps.citcat.com/en/translate";
        
        if (mIsXmlHttpBusy)
            return;

        if (mXmlHttp.readyState===0)
            mXmlHttp = new XMLHttpRequest();
            
        Foxlingo.session.translatedSearchTranslator = translator;
        Foxlingo.session.tempServiceName = serviceName;
        mIsXmlHttpBusy = true;
        Foxlingo.Translation.cancelTimeoutFunction = window.setTimeout(function() { Foxlingo.Translation.cancelTranslateRequest(); }, 40000);
        mXmlHttp.onreadystatechange = Foxlingo.Translation.getTranslateResponse;    
        mXmlHttp.open("POST", checkUrl, true);
        mXmlHttp.setRequestHeader("Content-Type", "application/x-www-form-urled");
    
        var pair;
        if (lang1=="en" && lang2=="ma")
            pair = "1"; 
        else if (lang1=="ma" && lang2=="en")
            pair = "2"; 
        else if (lang1=="en" && lang2=="ch")
            pair = "3";
        mXmlHttp.send("&src_text_=" + txt + "&trans_language_=" + pair);
        
    };
    
    pub.setTranslatedSearchIcon = function() { 
        if (!Foxlingo.Prefs.getBoolPref("translatedsearch") && document.getElementById("Foxlingo_translatedsearch-item")) {
            document.getElementById("Foxlingo_search-item").setAttribute("image", "chrome://foxlingo/skin/search.png"); 
            document.getElementById("Foxlingo_translatedsearch-item").setAttribute("tooltiptext", document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.translated.search") + ' - ' + document.getElementById("foxlingo-stringbundle").getString("foxlingo.text.off"));
        }
    };
    

    return pub;
}();
