/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Toolbar.Flags = function() {
    var pub = {};


    //public functions
    pub.populateToolbarFlags = function(istemp) {
        var prefValue;
        var toolbarflagsOption;

        for (var language in Foxlingo.Languages) {
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;
                
            if (istemp) {
                var toolbarFlagsElement = document.getElementById('toolbarflags');    
                if (toolbarFlagsElement)
                    toolbarflagsOption = toolbarFlagsElement.checked;
                else if (window.opener)
                    toolbarflagsOption = window.opener.document.getElementById('toolbarflags').checked;
                else 
                    toolbarflagsOption = false;
                prefValue = Foxlingo.session[language + "-toolbartemp"];
            } else {
                toolbarflagsOption = Foxlingo.Prefs.getBoolPref('toolbarflags');
                prefValue = Foxlingo.Prefs.getBoolPref(language + "-toolbar");    
            }
        
            var tempFlagButton = document.getElementById('Foxlingo_' + language + '-item');                

            if (prefValue && toolbarflagsOption) {
                if (!tempFlagButton && window.opener) { 
                    if (window.opener.document)
                        tempFlagButton = window.opener.document.getElementById('Foxlingo_' + language + '-item');
                    else
                        tempFlagButton = null;
                    if (window.opener.opener)
                        tempFlagButton = window.opener.opener.document.getElementById('Foxlingo_' + language + '-item');
                }
                
                if (tempFlagButton) {
                    tempFlagButton.style.display = "block";
                    tempFlagButton.setAttribute("collapsed", "false");
                }
            } else {
                if (!tempFlagButton) {
                    if (!window.opener || !window.opener.document)
                        tempFlagButton = null;
                    else
                        tempFlagButton = window.opener.document.getElementById('Foxlingo_' + language + '-item');
                }
                
                if (tempFlagButton) {
                    tempFlagButton.style.display = "none";
                    tempFlagButton.setAttribute("collapsed", "true");
                }
            }
            
            if (tempFlagButton)
                document.persist('Foxlingo_' + language + '-item', 'collapsed');
        }
        
        if (istemp) {
            if (istemp!='no')
                pub.toolbarflagsResize(null);
        }
        pub.toolbarflagsResize(null);
    };

    pub.initializeToolbarFlags = function() {
        const setFirstTimeToolbarFlagsFunc = "Foxlingo.Prefs.setCharPref('firsttimetoolbarflags', 'falsetemp');";
        const numFlagsPerColumn = 19;
        
        var flagsColumn = document.getElementById("options-toolbarflags");
        var strbundle = document.getElementById("foxlingo-stringbundle");
        var tempColumn = document.createElement("column");
        var countAvailableLanguages = 0;
        
        for (var language in Foxlingo.Languages) {
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;
                        
            if (!countAvailableLanguages) {
                var tempToggleItem = document.createElement("checkbox");
                
                tempToggleItem.setAttribute("id", "toggleall-toolbarflags");
                tempToggleItem.setAttribute("label", strbundle.getString("foxlingo.text.toggle.all"));
                var tempToggleItemCommand = "Foxlingo.Options.toggleAll('toolbarflags');";
                if (Foxlingo.Prefs.getCharPref('firsttimetoolbarflags')!='falsefinal')
                    tempToggleItemCommand += setFirstTimeToolbarFlagsFunc;
                tempToggleItem.setAttribute("oncommand", tempToggleItemCommand);
                tempColumn.appendChild(tempToggleItem);
            } else if (countAvailableLanguages%numFlagsPerColumn===0) {
                flagsColumn.appendChild(tempColumn);
                tempColumn = document.createElement("column");
                tempColumn.appendChild(document.createElement("spacer"));
            }            
            var tempItem = document.createElement("checkbox");
            
            tempItem.setAttribute("id", "toolbarflag-" + language);
            tempItem.setAttribute("label", strbundle.getString("foxlingo.language." + language));
            tempItem.setAttribute("style", "margin-bottom: -3px;");
            var flagLang = pub.getFlagLang(language);
                
            tempItem.setAttribute("src", "chrome://foxlingo/skin/flags-s/" + flagLang + ".png");
            tempItem.setAttribute("onclick", "Foxlingo.Toolbar.Flags.toggleToolbarFlag('" + language + "');");
            var tempItemCommand = "Foxlingo.Options.updateToggleAllButton('toolbarflags');";
            if (Foxlingo.Prefs.getCharPref('firsttimetoolbarflags')!='falsefinal')
                tempItemCommand += setFirstTimeToolbarFlagsFunc;
            tempItem.setAttribute("oncommand", tempItemCommand);
            
            if (Foxlingo.Prefs.getCharPref("firsttimetoolbarflags")=="true") {
                var checkValue = window.opener.document.getElementById(language + '-langfrom').checked;
                Foxlingo.session[language + "-toolbartemp"] = checkValue;
            }
            
            if (Foxlingo.session[language + "-toolbartemp"]) {
                tempItem.setAttribute("checked", "true");
            }
            tempColumn.appendChild(tempItem);
                        
            countAvailableLanguages++;
        }
        
        if (countAvailableLanguages%numFlagsPerColumn!==0)
            flagsColumn.appendChild(tempColumn);    
        
        var flagSizeIndex;
        var flagSizeObject = document.getElementById("iconsizefl");
        var iconsizefl = Foxlingo.Prefs.getCharPref("iconsizefl");
        if (iconsizefl=="smallflags")
            flagSizeIndex = 0;
        else if (iconsizefl=="mediumflags")
            flagSizeIndex = 1;
        else
            flagSizeIndex = 2;
        flagSizeObject.selectedIndex = flagSizeIndex;    
        Foxlingo.Options.updateToggleAllButton('toolbarflags');
    };
    
    pub.toggleToolbarFlag = function(button, boolVal) {
        if (!window.opener.opener)
            return;
        
        if (boolVal)
            Foxlingo.session[button + "-toolbartemp"] = button;
        else
            Foxlingo.session[button + "-toolbartemp"] = !Foxlingo.session[button + "-toolbartemp"];
    };

    pub.updateflagsOverflowMenu = function() {
        var overflowMenu = document.getElementById("Foxlingo_flagsOverflowMenu");
            
        for (var language in Foxlingo.Languages) {
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;
                
            var button = document.getElementById("Foxlingo_" + language + '-item');
            if (!button)
                continue;
                
            var menu = document.getElementById("Foxlingo_" + language + "-itemoverflow");
            menu.collapsed = !(Foxlingo.Prefs.getBoolPref(language + '-toolbar') && button.collapsed);
        }
    };
    
    pub.changeIconSize = function(size) {
        Foxlingo.Toolbar.toolattr.setAttribute("iconsizefl", size);
    };

    pub.removeAllChildrenFromToolbarFlags = function() { 
        for (var language in Foxlingo.Languages) { 
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;
                 
            var tempItem = document.getElementById("Foxlingo_" + language);
            if (!tempItem && window.opener && window.opener.document)
                tempItem = window.opener.document.getElementById("Foxlingo_" + language);
                
            if (tempItem)
                Foxlingo.Utils.removeChildren(tempItem); 
        } 
    };

    pub.toolbarflagsResize = function(event) {
        var overflowMenu = document.getElementById("Foxlingo_flagsOverflowMenu");
        var chevron = document.getElementById("Foxlingo_flagsOverflowButton");
        var available = window.innerWidth;
        var documentBase = document;
        
        if (!overflowMenu) {
            if (!window.opener)
                return;
            overflowMenu = window.opener.document.getElementById("Foxlingo_flagsOverflowMenu");
            chevron = window.opener.document.getElementById("Foxlingo_flagsOverflowButton");
            available = window.opener.innerWidth;
            documentBase = window.opener.document;
        }
        if (!overflowMenu && window.opener.opener) {
            overflowMenu = window.opener.opener.document.getElementById("Foxlingo_flagsOverflowMenu");
            chevron = window.opener.opener.document.getElementById("Foxlingo_flagsOverflowButton");
            available = window.opener.opener.innerWidth;
            documentBase = window.opener.opener.document;
        }

        if (!overflowMenu || !chevron)
            return;

        var chevronWidth = 20;
        var overflowed = false;
        var j = 0;
    
        for (var language in Foxlingo.Languages) {
            if (!Foxlingo.isAvailableLanguageFrom(language))
                continue;

            var flag = documentBase.getElementById("Foxlingo_" + language + "-item");
            if (!flag || !flag.boxObject)
                continue;
            
            if (!flag.collapsed)
                j++;
            
            flag.collapsed = overflowed;
            if (Foxlingo.Toolbar.isOverflowed(flag, chevronWidth, available)) {
                overflowed = true; 
                flag.collapsed = true;
                if (chevron.collapsed)
                    chevron.collapsed = false;
            }
        }
        if (j===0)
            chevron.collapsed = true;
        else if (chevron.collapsed==overflowed)
            chevron.collapsed = !chevron.collapsed;
    };
    
    pub.cancelToolbarFlags = function() {
        Foxlingo.Toolbar.toolattr.setAttribute("iconsizefl", Foxlingo.Prefs.getCharPref("iconsizefl"));
        for (var language in Foxlingo.Languages) {
            if (Foxlingo.isAvailableLanguageFrom(language))
                pub.toggleToolbarFlag(language, Foxlingo.Prefs.getBoolPref(language + "-toolbar"));
        }
        pub.populateToolbarFlags();
    };
    
    pub.getFlagLang = function(language) {
        var flagLang = language;
        if ((language=='english' && Foxlingo.Prefs.getCharPref("engflag")!="usflag") ||
            (language=='french' && Foxlingo.Prefs.getCharPref("freflag")!="frflag") ||
            (language=='spanish' && Foxlingo.Prefs.getCharPref("spaflag")!="spflag"))
            flagLang += "2";
        return flagLang;
    };


    return pub;
}();
