/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Toolbar = function() {
    var pub = {};

    //public properties
    //'defines'
    pub.TOOLBAR_VERSION = "2-4-3c";

    //constants
    pub.ToolbarButtons = ['searchbox', 'webtranslation', 'texttranslation', 'dictionary', 'services', 'grammar', 'backanddomain', 'translatedsearch', 'options', 'highlighter',
                          'searchwordbuttons'];

    //public members
    pub.toolattr;

    //private properties
    var mCurrentWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator).getMostRecentWindow("navigator:browser");


    //public functions
    pub.init = function() {
        pub.toolattr = mCurrentWindow.document.getElementById("foxlingo-toolbar-" + Foxlingo.Toolbar.TOOLBAR_VERSION);
    };

    pub.changeDomainButton = function(target, isReferer) {
        var translator;

        if (!Foxlingo.Utils.isBrowserWindow())
            return;
        
        var backanddomain = document.getElementById("Foxlingo_backanddomain-item");
        if (!backanddomain) 
            return;
        
        var thisDoc = target;
        if (!thisDoc)
            thisDoc = Foxlingo.Utils.getCurrentDocument();
        if (!thisDoc)
            return;
            
        if (target && target.URL!=document.getElementById("content").selectedBrowser.contentWindow.document.URL)
            return;

        if (Foxlingo.session.currentDomainButtonProperties.url && Foxlingo.session.currentDomainButtonProperties.url==thisDoc.URL)
            return;
        Foxlingo.session.currentDomainButtonProperties.url = thisDoc.URL;

        var langfrom = Foxlingo.Utils.getDomain("language", null, thisDoc);
        var siteLanguage = Foxlingo.Utils.getSiteLanguage(isReferer);
	if (Foxlingo.Prefs.getBoolPref("identifysitelanguage") && siteLanguage && siteLanguage!="unknown" && siteLanguage!="undefined")
	    langfrom = siteLanguage;

        var strbundle = document.getElementById("foxlingo-stringbundle");
        var langto = Foxlingo.Prefs.getCharPref("options-dlang");
        if (!langto)
            langto = "english";
        var autotranslate = Foxlingo.Prefs.getBoolPref("autotranslate");
        
        var realDomain = Foxlingo.Utils.getDomain("domain");
        
        //autotranslate if needed
        if (autotranslate && !Foxlingo.Translation.isUrlTranslated(thisDoc) && !Foxlingo.Translation.Auto.findBackList(thisDoc.URL) &&
            !Foxlingo.Translation.Auto.findRemoveSentList(thisDoc.URL)) {
            translator = Foxlingo.Translation.getTranslator(langfrom, langto);
            Foxlingo.Translation.Auto.alreadySentList.push(thisDoc.URL);
            if (realDomain=="unknowndomain" && translator=="google") {
                Foxlingo.Translation.Auto.autoTranslate("auto", langto, translator, null, thisDoc);
            } else if (realDomain!="unknowndomain" && translator!="none") {
                Foxlingo.Translation.Auto.autoTranslate(langfrom, langto, translator, null, thisDoc);
            }
        }

        if (Foxlingo.session.currentDomainButtonProperties.domain && Foxlingo.session.currentDomainButtonProperties.domain==realDomain)
            return;
        Foxlingo.session.currentDomainButtonProperties.domain = realDomain;

        var domainLabel = strbundle.getString("foxlingo.domain." + realDomain);
            
        var httpsDomain;
        if (Foxlingo.Utils.getUrl(null, target).substr(0,6)=="ftp://")
            httpsDomain = Foxlingo.Utils.getDomain("domain", null, null, true);
        
        var iconDomain = realDomain;
        if (httpsDomain)
            iconDomain = httpsDomain;
        backanddomain.setAttribute("image", "chrome://foxlingo/skin/domains/" + iconDomain + ".png");
        
        var labelStr = '';
        if (realDomain=="back")
            labelStr = "original";
        else
            labelStr = "autotrans";
        backanddomain.setAttribute("label", strbundle.getString("foxlingo.text." + labelStr));
        
        var isCurrentLanguages = false;
        if (Foxlingo.session.currentDomainButtonProperties.langTo && Foxlingo.session.currentDomainButtonProperties.langTo==langto &&
            Foxlingo.session.currentDomainButtonProperties.langFrom==langfrom)
            isCurrentLanguages = true;
        Foxlingo.session.currentDomainButtonProperties.langTo = langto;
        Foxlingo.session.currentDomainButtonProperties.langFrom = langfrom;

        var isDisabled = "false"; 
        var space = strbundle.getString("foxlingo.space").replace(/&nbsp;/g," ").replace(/_/g," ");
        var tooltipTextStr = domainLabel;
        if (realDomain=="disabled") {
            if (httpsDomain)
                tooltipTextStr = strbundle.getString("foxlingo.domain." + httpsDomain) + ' - ' +
                                 strbundle.getString("foxlingo.text.ERROR_HTTPS_FTP");
            else
                tooltipTextStr = strbundle.getString("foxlingo.text.ERROR_HTTPS_FTP");
            isDisabled = "true"; 
        } else if (realDomain!="unknowndomain" && realDomain!="back") {
            var restTooltipTextStr = "";         
            if (!autotranslate || realDomain=="back" || Foxlingo.Translation.Auto.findBackList(thisDoc.URL))
                backanddomain.setAttribute("disabled", "false");
            
            var langfromForDomainTooltip = "";
            if (realDomain=="uk" || realDomain=="us" || langfrom!="english")
                langfromForDomainTooltip = strbundle.getString("foxlingo.openp") + strbundle.getString("foxlingo.language." + langfrom) + strbundle.getString("foxlingo.closep");
            translator = Foxlingo.Translation.getTranslator(langfrom, langto);
            if (translator=="google" && !Foxlingo.Prefs.getBoolPref("autotranslate"))
                langfrom = "auto";
            var autoPair = Foxlingo.Translation.Auto.getAutoPair(langfrom, langto, translator);
            var langfromLabel = strbundle.getString("foxlingo.language." + langfrom);
            var translate = strbundle.getString("foxlingo.text.translate");
            if (autoPair=="none") {
                restTooltipTextStr += strbundle.getString("foxlingo.text.ERROR_DOMAIN_SAME_LANGUAGE") + space; 
                isDisabled = "true";
            } else {
                restTooltipTextStr += translate + space;
                if (autoPair=="english" && translator!="google") 
                    restTooltipTextStr += strbundle.getString("foxlingo.language.english");
                else 
                    restTooltipTextStr += langfromLabel;
                restTooltipTextStr += strbundle.getString("foxlingo.to").replace(/&nbsp;/g," ").replace(/_/g," ");
            }    
            restTooltipTextStr += strbundle.getString("foxlingo.language." + langto);
            Foxlingo.session.currentDomainButtonProperties.restTooltipTextStr = restTooltipTextStr;
            tooltipTextStr = restTooltipTextStr + "\n" + strbundle.getString("foxlingo.text.domain") + ":" + space + domainLabel + space + langfromForDomainTooltip;
        }
        backanddomain.setAttribute("tooltiptext", tooltipTextStr);
        backanddomain.setAttribute("disabled", isDisabled);         
    };

    pub.toggleToolbar = function() {
        var toolbar = document.getElementById("foxlingo-toolbar-" + pub.TOOLBAR_VERSION);
        toolbar.collapsed = !toolbar.collapsed;
        document.persist("foxlingo-toolbar-" + pub.TOOLBAR_VERSION, "collapsed");
    };

    pub.setToolbar = function(boolVal) {
        var toolbar = document.getElementById("foxlingo-toolbar-" + pub.TOOLBAR_VERSION);
        toolbar.collapsed = boolVal;
        document.persist("foxlingo-toolbar-" + pub.TOOLBAR_VERSION, "collapsed");
    };

    pub.onButtonMouseover = function(button1) {
        if (!button1 || button1.open)
            return;
            
        var button2 = null;
        var buttons = button1.parentNode.getElementsByTagName("toolbarbutton");        
        
        for (var i = 0; i < buttons.length; i++) {
            button2 = buttons.item(i);
            if (button2 && button2!=button1 && button2.open) {
                button2.open = false;
                button1.open = true;
                break;
            }
        }
    };
    
    pub.resizerMouseDown = function(event) {
        Foxlingo.session.dragging = true;
        Foxlingo.session.lastX = event.screenX;
        Foxlingo.session.isChangedSearchboxSize = true;
    };

    pub.resize = function(isCheckResizer) {
        var buttons = document.getElementById("Foxlingo_searchwordbuttons-item");
        
        if (!buttons)
            return;
        
        var overflowMenu = document.getElementById("Foxlingo_Overflow-Menu");
        var chevron = document.getElementById("Foxlingo_Overflow-Button");
        
        if (!chevron)
            return;
        
        var chevron2 = document.getElementById("Foxlingo_flagsOverflowButton");
        var chevronWidth = 25;
        
        var chevron2Width;
        if (!chevron2 || !chevron2.boxObject)
            chevron2Width = 0;
        else
            chevron2Width = chevron2.boxObject.width;
        
        var available = window.innerWidth;
        var overflowed = false;
        
        for (var i =0; i <buttons.childNodes.length; i++) {
            var button = buttons.childNodes[i];
            if (button && button.id=="Foxlingo_wordfindCustomize" || !button.boxObject)
                continue;
            
            var offset = button.boxObject.x;
            button.collapsed = overflowed;
            if (pub.isOverflowed(button, chevronWidth + chevron2Width, available)) {
                overflowed = true; 
                if (!button.collapsed)
                    button.collapsed = true;
                if (chevron.collapsed)
                    chevron.collapsed = false;
            }
        }
        
        if (chevron.collapsed==overflowed)
            chevron.collapsed = !chevron.collapsed;
        if (Foxlingo.Prefs.getBoolPref("toolbarflags"))
            Foxlingo.Toolbar.Flags.populateToolbarFlags();
        else
            Foxlingo.Toolbar.Flags.toolbarflagsResize();
            
        if (!isCheckResizer)
            Foxlingo.globalMouseMove(null, true); 
    };
    
    pub.toggleToolbarButton = function(button) {
        var windowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
        var enumerator = windowMediator.getEnumerator("");
        while(enumerator.hasMoreElements()) {
            var win = enumerator.getNext();
            var toolbarbutton = win.document.getElementById('Foxlingo_' + button + '-item');
            if (toolbarbutton)
                toolbarbutton.collapsed = !toolbarbutton.collapsed;
        }
    };
    
    pub.toggleToolbarButtonsText = function(hardValue) {
        var isShowbuttontext = hardValue;
        if (!isShowbuttontext && document.getElementById("showbuttontext"))
            isShowbuttontext = document.getElementById("showbuttontext").checked;
        
        var tempClass = (isShowbuttontext ? 'always' : 'never');    
                
        for each (var toolbarButton in pub.ToolbarButtons) {        
            if (toolbarButton=='translatedsearch')
                continue;
                
            var tempButton = document.getElementById('Foxlingo_' + toolbarButton + '-item');
            if (!tempButton || !window.opener)
                continue;

            if (toolbarButton!='searchwordbuttons') 
                tempButton.setAttribute("foxlingo-show-text", tempClass);
        }
    };
    
    pub.fullLogo = function() {
        var fullLogo = !Foxlingo.Prefs.getBoolPref("fulllogo");
        pub.toolattr.setAttribute("fulllogo", fullLogo);
        Foxlingo.Prefs.setBoolPref("fulllogo", fullLogo);
    };
    
    pub.updateToolbarButtons = function() {
        for each (var toolbarButton in pub.ToolbarButtons) {
            var tempButton = document.getElementById('Foxlingo_' + toolbarButton + '-item');
            if (!tempButton && window.opener && window.opener.document)
                tempButton = window.opener.document.getElementById('Foxlingo_' + toolbarButton + '-item');
            if (tempButton)
                tempButton.collapsed = !Foxlingo.Prefs.getBoolPref('button-' + toolbarButton);
        }    
    };
    
    pub.updateToolbarButtonsText = function() {
        for each (var toolbarButton in pub.ToolbarButtons) {
            if (toolbarButton=='translatedsearch')
                continue;
            
            var tempButton = document.getElementById('Foxlingo_' + toolbarButton + '-item');    
            if (!tempButton) {            
                if (window.opener) 
                    tempButton = window.opener.document.getElementById('Foxlingo_' + toolbarButton + '-item');
            }

            if (!tempButton)
                continue;
                
            var tempClass = (Foxlingo.Prefs.getBoolPref('showbuttontext') ? 'always' : 'never');
            
            if (toolbarButton!='searchwordbuttons')
                tempButton.setAttribute("foxlingo-show-text", tempClass);    
        }
    };
    
    pub.doResize = function(opt_event) {
        return; //hack!
        var searchBox = document.getElementById("Foxlingo_searchbox-item");
        if (!searchBox)
            return;
        var searchItem = document.getElementById("Foxlingo_search-item");
        var toolbar = document.getElementById("foxlingo-toolbar-" + pub.TOOLBAR_VERSION);
        var sbStyle = document.defaultView.getComputedStyle(searchBox, "");
        var currentWidth = parseInt(sbStyle["width"], 10);
        
        if (toolbar && searchItem && currentWidth>0) {
            if (searchBox.parentNode==toolbar) {
                if (Foxlingo.Utils.isFF) {
                    toolbar.removeChild(searchBox);
                    toolbar.insertItem("Foxlingo_searchbox-item", searchItem, null, false);
                } else {
                    var tempChild = searchBox.cloneNode(true);
                    toolbar.replaceChild(tempChild, searchBox);
                }
                window.setTimeout(function() { var searchBox = document.getElementById("Foxlingo_searchbox-item"); if (searchBox) searchBox.setAttribute("width", ' + currentWidth + '); }, 0);
            }
            Foxlingo.Search.loadSearchHistory();
            Foxlingo.Customize.applyStyles();
        }
        Foxlingo.Search.clearSearchWordButtons();
        Foxlingo.Search.termsHaveUpdated();
    };

    pub.isOverflowed = function(element, chevronWidth, available) {
        var offset = element.boxObject.x;

        return ((Foxlingo.direction=='ltr' && offset + element.boxObject.width + chevronWidth > available) ||
                (Foxlingo.direction!='ltr' && offset - chevronWidth < 0));
    };


    return pub;
}();
