/*
 * Version: MPL v. 1.1
 *
 * The contents of this file are subject to the MPL License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/MPL-1.1.txt
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the FoxLingo.
 *
 * The Initial Developer of the Original Code is
 * Linkular LLC
 *
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * */

Foxlingo.Translation = function() {
    var pub = {};

    //public properties
    //constants
    pub.Translators = ['google', 'worldlingo', 'apertium', 'bingtranslator', 'cervantes', 'citcat', 'elmundo', 'excite', 'freetranslation', 'gencat', 'gramtrans',
                       'huajian', 'internostrum', 'interpret', 'intertran', 'lingro', 'nifty', 'ocn', 'opentrad', 'paktranslations',
                       'promt', 'sli', 'statmt', 'toggletext', 'traduku', 'translatica', 'vilnet', 'websphere', 'yahoo']; 

    //public members
    pub.cancelTimeoutFunction;


    //private properties
    //private members
    var mXmlHttpTS = new XMLHttpRequest(); 
    var mIsTSxmlHttpBusy = false;
    var mTSQueue = [];


    //public functions
    pub.cancelTranslateRequest = function() {
        mIsTSxmlHttpBusy = false;
        mXmlHttpTS.abort();
        var strbundle = document.getElementById("foxlingo-stringbundle");
        Foxlingo.Status.showStatusMessage(strbundle.getString("foxlingo.text.translation.timeout"));
        
        if (Foxlingo.session.tempServiceName) 
            Foxlingo.openSite(Foxlingo.session.tempServiceName, Foxlingo.session.translatedSearchTerms, null, target);
        else 
            Foxlingo.Search.search(Foxlingo.session.translatedSearchTerms, Foxlingo.session.translatedSearchType, (Foxlingo.session.translatedSearchTerms.length===0), null); 
    };
    
    pub.translateUrl = function(language1, language2, translator, target) {
        var url = Foxlingo.Utils.getUrl(null, target);

        if (url=="about:blank") {
            Foxlingo.Utils.openMessage("ERROR_EMPTY_PAGE"); 
            return;
        }
        
        if (pub.isUrlTranslated(target))
            url = pub.backToOriginal(target);
        
        if (url && url.substr(0,6)=="ftp://" && Foxlingo.Prefs.getBoolPref("autotranslate"))  
            return;
    
        if (target)
            target.beingTranslated = true;
        
        var lang1 = pub.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(language1, "shortName"), translator);
        var lang2 = pub.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(language2, "shortName"), translator);
        var newUrl = pub.getTranslationUrl(translator, lang1, lang2, url);

//        if (newUrl)
//            newUrl = newUrl.replace(/\%3D/g,"=").replace(/\%3F/g,"?");

        if (translator=="interpret")
            return;

        if (target && target.location)
            target.location.href = newUrl;            
        else if (!target && !Foxlingo.Prefs.getBoolPref("autotranslate"))
            Foxlingo.Utils.openTabT(newUrl);
        else
            Foxlingo.Utils.openURL(newUrl, "currenttab");
    };
    
    pub.openSiteTranslated = function(serviceName) {
        if (!Foxlingo.Prefs.getBoolPref("translatedsearch")) {
            Foxlingo.openSite(serviceName, "textbox");
            return;
        }
        var langfrom = Foxlingo.Prefs.getCharPref("translatedsearchlangfrom");
        var langto = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
        var translator = pub.getTranslator(langfrom, langto);
        var strbundle = document.getElementById("foxlingo-stringbundle");
        
        window.setTimeout(function() { Foxlingo.Status.showStatusMessage(strbundle.getString("foxlingo.text.translated.search.wait")); }, 100);
        Foxlingo.Translation.Text.translateText(langfrom, langto, translator, serviceName);
    };
    
    pub.isUrlTranslated = function(target) {
        return (Foxlingo.Utils.getUrl(null, target)!=pub.backToOriginal(target));
    };
    
    pub.getTranslator = function(language1, language2) {
        if (language1=="disabled" || language1=="unknowndomain")
            return "none";
        var langfrom = Foxlingo.Utils.changeLanguage(language1, "shortName");
        var langto = Foxlingo.Utils.changeLanguage(language2, "shortName");
        
        for each (var translator in pub.Translators) {
            if (translator!="google" && translator!="worldlingo" && langfrom==langto)
                return "none";
            if (doesTranslatorSupportLanguages(translator, langfrom, langto))
                return translator;
                
        }
        if (pub.getTranslatorArray("google").indexOf(langto)!=-1)
            return "google"; 
        return "none";
    };
    
    pub.sendTranslateRequest = function(urlToSend, translator, serviceName, fromQueue,injectionProperties) {
        var addToQueue = false;
        if (mIsTSxmlHttpBusy && !fromQueue) {
            if (findTranslateRequest(urlToSend))
                return;
            var currentTab = Foxlingo.Utils.getCurrentDocument();
            if (!serviceName)
                serviceName = "";
            var queueLangTo = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
            var queueLangFrom = Foxlingo.Prefs.getCharPref("translatedsearchlangfrom");
            var queueItem = Foxlingo.session.translatedSearchTerms + ";" + queueLangFrom + ";" + queueLangTo + ";" + serviceName + ";" + translator + ";" + Foxlingo.session.translatedSearchType;
            mTSQueue.push({url: urlToSend, item: queueItem, tab: currentTab});
            return;
        }
        mIsTSxmlHttpBusy = true;
        var langFrom, langTo, thisService, thisTranslator, thisUrl, thisTab;
        if (fromQueue) {
            var item = mTSQueue.pop();
            var thisQueueItem = item.item.split(";");
            var searchTerms = thisQueueItem[0];
            langFrom = thisQueueItem[1];
            langTo = thisQueueItem[2];
            thisService = thisQueueItem[3];
            thisTranslator = thisQueueItem[4];
            var thisSearchType = thisQueueItem[5];
            thisUrl = item.url;
            thisTab = item.tab;
            Foxlingo.session.translatedSearchTerms = searchTerms;
            Foxlingo.session.translatedSearchType = thisSearchType;
        } else {
            langFrom = Foxlingo.Prefs.getCharPref("translatedsearchlangfrom");
            langTo = Foxlingo.Prefs.getCharPref("translatedsearchlangto");
            thisService = serviceName;
            thisTranslator = translator;
            thisUrl = urlToSend;
            thisTab = Foxlingo.Utils.getCurrentDocument();        
        }
        Foxlingo.session.translatedSearchTranslator = thisTranslator;
        mXmlHttpTS.onreadystatechange = function sendTargetTranslateResponse() {
                if (injectionProperties) {
                    var textElements = injectionProperties["elements"];
                    var counter = 0;
                    var translatedArray = Foxlingo.Translation.getTranslateResponse(thisTab,true)
                    if (!translatedArray)
                        return;
                    translatedArray += " ";
                    translatedArray = translatedArray.split(injectionProperties["separator"]);
                    Foxlingo.Utils.debug(translatedArray.length);
                    for (var element in textElements) {
                            if (textElements[element].innerHTML) {
                                Foxlingo.Utils.debug(counter);
                                textElements[element].innerHTML = translatedArray[counter];
                                counter++;
                            }
                        }
                    }
                else 
                    Foxlingo.Translation.getTranslateResponse(thisTab);
            };
        
        Foxlingo.session.translatedSearchTranslator = thisTranslator;
        Foxlingo.session.tempServiceName = thisService;
        
        pub.cancelTimeoutFunction = window.setTimeout(function() { Foxlingo.Translation.cancelTranslateRequest(); }, 40000);
        
        if (thisTranslator=="worldlingo") {        
            var lang1 = pub.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(langFrom, "shortName"), thisTranslator);
            var lang2 = pub.changeLanguagesToCode(Foxlingo.Utils.changeLanguage(langTo, "shortName"), thisTranslator);
            var url = "http://www.worldlingo.com/S1889.1/api";
            var query = "wl_data=" + Foxlingo.session.translatedSearchTerms + "&wl_srclang=" + lang1 + "&wl_trglang=" + lang2 + "&wl_password=secret";
            thisUrl = url + "?" + query;
        }
        mXmlHttpTS.open("GET", thisUrl, true);
        mXmlHttpTS.send(null);
    };
    
    pub.getTranslateResponse = function(target, isReturnTranslatedText) {
        if (mXmlHttpTS.readyState!=4 && mXmlHttpTS.readyState!="complete")
            return null;
        var counter = 0;
        mIsTSxmlHttpBusy = false;
        window.clearTimeout(pub.cancelTimeoutFunction);
        var responseText = mXmlHttpTS.responseText;
        var translatedText;
        switch (Foxlingo.session.translatedSearchTranslator) {
            case "google":
                translatedText = responseText.match(/short_text"><[^>]*>([^<]*)</i);
                if (translatedText && translatedText[1])
                    translatedText = translatedText[1];
                else {
                    translatedText = responseText.match(/"long_text">(<span.*<\/span>)<\/span><\/div><\/div><div id=gt-res-tools/i);
                    if (translatedText && translatedText[1]) {
                        translatedText = translatedText[1];
                        var tmp = document.createElement("DIV");
                        tmp.innerHTML = translatedText;
                        var spans = tmp.getElementsByTagName("span");
                        for (var i = 0; i < spans.length; i++) {
                            translatedText += spans[i].replace(/(<([^>]+)>)/ig,"");;
                        }
                        translatedText = translatedText.replace(/(<([^>]+)>)/ig,"");
                    }
                }
                break;
            case "worldlingo": 
                if (responseText && responseText.split("\n"))
                    translatedText = responseText.split("\n")[1];                
                break;            
            case "apertium":
                translatedText = responseText;
                break;
            case "intertran": 
                if (responseText)
                    responseText = responseText.substr(responseText.indexOf('name="translation"') + 19);
                if (responseText)
                    translatedText = responseText.substr(1, responseText.indexOf("</textarea>")-1);
                break;    
            default: break;
        }
        if (Foxlingo.session.translatedSearchTranslator!='worldlingo' && translatedText) {
            translatedText = translatedText.replace(/^\s+/, '');
            translatedText = translatedText.replace(/\s+$/, '');
        }
        if (translatedText && isReturnTranslatedText)
            return translatedText;
        else if (translatedText) {
            translatedText = encodeURIComponent(translatedText);
            if (Foxlingo.session.tempServiceName) 
                Foxlingo.openSite(Foxlingo.session.tempServiceName, translatedText, null, target);
            else 
                Foxlingo.Search.search(translatedText, Foxlingo.session.translatedSearchType, (translatedText.length===0), target, true); 
        } else if (Foxlingo.session.tempServiceName) {
            Foxlingo.openSite(Foxlingo.session.tempServiceName, Foxlingo.session.translatedSearchTerms, null, target);
        } else {
            Foxlingo.Search.search(Foxlingo.session.translatedSearchTerms, Foxlingo.session.translatedSearchType, (Foxlingo.session.translatedSearchTerms.length===0), target, true); 
        }
        if (mTSQueue.length!==0)
            pub.sendTranslateRequest(null, null, null, true);
        return null;
    };
    
    pub.getTranslatorArray = function(translator) {
        switch (translator) {
            case "worldlingo": return ['en', 'ar', 'cn', 'tw', 'nl', 'fr', 'de', 'el', 'it', 'ja', 'ko', 'pt', 'ru', 'es', 'sv'];
            case "google":
                return ['xx', 'en', 'af', 'sq', 'ar', 'hy', 'az', 'eu', 'be', 'bu', 'ca', 'cn', 'tw', 'cr', 'cz', 'da', 'nl', 'et', 'ph', 'fi', 'fr', 'gl', 'ka', 'de', 'el', 'ht', 'he', 'hi', 'hu', 'ic', 'in', 'ir', 'it',
                        'ja', 'ko', 'lv', 'lt', 'mk', 'ma', 'mt', 'no', 'pe', 'po', 'pt', 'ro', 'ru', 'se', 'sk', 'sl', 'es', 'sw', 'sv', 'th', 'tu', 'uk', 'ur', 'vi', 'we', 'yi'];
            case "freetranslation": return ['en-nl', 'en-fr', 'en-de', 'en-it', 'en-no', 'en-pt', 'en-es', 'nl-en', 'fr-en', 'de-en', 'de-fr', 'it-en', 'pt-en', 'es-en'];
            case "imtranslator":
                return ['en-fr', 'en-de', 'en-pt', 'en-ru', 'en-es', 'fr-en', 'fr-de', 'fr-ru', 'fr-es', 'de-en', 'de-ru',
                        'it-ru', 'pt-en', 'ru-en', 'ru-fr', 'ru-de', 'ru-es', 'es-en', 'es-fr', 'es-ru'];
            case "websphere":
                return ['en-cn', 'en-tw', 'en-fr', 'en-de', 'en-it', 'en-ja', 'en-ko', 'en-pb', 'en-es', 'cn-en', 'tw-en',
                        'fr-en', 'de-en', 'it-en', 'ja-en', 'es-en'];
            case "gencat": return ['en-ca', 'fr-ca', 'de-ca', 'es-ca', 'ca-en', 'ca-fr', 'ca-de', 'ca-es'];
            case "interpret":
                return ['en-af', 'af-en', 'en-nl', 'en-fr', 'en-de', 'en-in', 'en-it', 'en-ja', 'en-ko', 'en-pt', 'en-es', 'en-sv', 'en-no', 'ar-en',
                        'nl-en', 'de-en', 'fr-en', 'fr-nl', 'fr-de', 'de-fr', 'it-en', 'it-en', 'ja-en', 'ko-en', 'pt-en', 'es-en'];
            case "intertran":
                return ['en', 'bu', 'cr', 'cz', 'da', 'nl', 'ph', 'fi', 'fr', 'de', 'el', 'hu', 'ic', 'it', 'ja', 'la', 'no', 'po', 'pt', 'pb', 'ro',
                        'ru', 'se', 'sl', 'es', 'sa', 'sv', 'tu', 'we'];
            case "toggletext": return ['en-in', 'in-en'];
            case "lingro": return ['en-es', 'es-en', 'fr-en', 'en-fr', 'en-de', 'de-en', 'en-it', 'it-en', 'en-po', 'po-en', 'en-sw', 'sw-en', 'sw-po', 'po-sw'];
            case "bingtranslator": return ['xx', 'en', 'ar', 'cn', 'tw', 'cz', 'da', 'nl', 'fr', 'de', 'el', 'he', 'it', 'ja', 'ko', 'po', 'pt', 'ru', 'es', 'sv', 'th'];
            case "traduku": return ['en-eo', 'hu-eo', 'eo-en', 'eo-hu'];
            case "internostrum": return ['ca-es', 'es-ca'];
            case "sli": return ['gl-es', 'es-gl', 'gl-pt', 'pt-gl'];
            case "opentrad": return ['ca-es', 'gl-es', 'es-eu', 'es-ca', 'es-gl'];
            case "apertium":
                return ['af-en', 'as-es', 'br-fr', 'ca-en', 'ca-eo', 'ca-es', 'ca-fr', 'ca-oc', 'ca-oi', 'ca-pt', 'ca-ro', 'we-en', 'da-sv', 'de-en', 'en-af', 'en-bn', 'en-ca', 'en-we', 'en-de', 'en-eo', 'en-es', 'en-gl', 'en-ne', 'en-po', 'eo-en', 'es-as', 'es-ca', 'es-va', 'es-en', 'es-eo', 'es-fr', 'es-gl', 'es-it', 'es-oi', 'es-oc', 'es-pt', 'es-pb', 'es-ro', 'eu-es', 'pe-tg', 'fr-ca', 'fr-es', 'gd-ir', 'gl-en', 'gl-es', 'gl-pt', 'ic-en', 'it-ca', 'it-es', 'no-nn', 'nn-no', 'oi-ca', 'oi-es', 'oc-ca', 'oc-es', 'po-en', 'pt-ca', 'pt-es', 'pt-gl', 'ro-ca', 'ro-es', 'sv-da', 'tg-pe', 'sc-mk', 'mk-sb', 'mk-sh', 'mk-ss'];
            case "ask": return ['en', 'fr', 'de', 'he', 'it', 'ja', 'pt', 'ru', 'es'];
            case "cervantes": return ['en-es', 'ca-es', 'fr-es', 'pt-es', 'es-en', 'es-ca', 'es-fr', 'es-pt', 'gl-es', 'es-gl'];
            case "paktranslations": return ['en-ur'];
            case "promt":
                return ['en-fr', 'en-de', 'en-pt', 'en-ru', 'en-es', 'fr-en', 'fr-de', 'fr-ru', 'fr-es', 'de-en', 'de-fr', 'de-ru', 'de-es', 'it-en', 'it-ru', 'pt-en',
                        'ru-en', 'ru-fr', 'ru-de', 'ru-es', 'es-en', 'es-fr', 'es-de', 'es-ru'];
            case "yahoo":
                return ['tw-cn', 'cn-tw', 'en-cn', 'en-tw', 'en-nl', 'en-fr', 'en-de', 'en-el', 'en-it', 'en-ja', 'en-ko', 'en-pt', 'en-ru', 'en-es', 'cn-en', 'tw-en',
                        'nl-en', 'nl-fr', 'fr-en', 'fr-nl', 'fr-de', 'fr-el', 'fr-it', 'fr-pt', 'fr-ru', 'fr-es', 'de-en', 'de-fr', 'el-en', 'el-fr', 'it-en', 'it-fr',
                        'ja-en', 'ko-en', 'pt-en', 'pt-fr', 'ru-en', 'es-en', 'es-fr'];
            case "excite": return ['en-ja', 'cn-ja', 'ja-en', 'ja-cn', 'ja-ko', 'ko-ja'];
            case "ocn": return ['en-ja', 'cn-ja', 'ja-en', 'ja-cn', 'ja-ko', 'ko-ja'];
            case "nifty": return ['en-ja', 'cn-ja', 'ja-en', 'ja-cn', 'ja-ko', 'ko-ja'];
            case "gramtrans": return ['en-da', 'en-eo', 'en-no', 'en-sv', 'da-en', 'da-eo', 'da-no', 'da-sv', 'no-da', 'no-en', 'no-sv', 'sv-da', 'sv-en', 'sv-no'];
            case "huajian": return ['en-cn', 'cn-en', 'ja-cn', 'cn-ja'];
            case "vilnet": return ['en-ja', 'ja-en'];
            case "citcat": return ['en-ma', 'ma-en', 'en-cn'];
            case "elmundo": return ['en-es', 'es-en', 'fr-es', 'es-fr', 'en-de', 'de-en'];
            case "translatica" : return ['en-po', 'po-en', 'de-po', 'po-de', 'ru-po', 'po-ru'];
            case "statmt": return ['en-de', 'es-en', 'de-en', 'fr-en'];
            default: return "none";
        }
    };
    
    pub.isPairBeta = function(langfrom, langto, translator, isLongLangs) {
        if (translator!="apertium")
            return false;
            
        if (isLongLangs) {
            langfrom = Foxlingo.Utils.changeLanguage(langfrom, "shortName");
            langto = Foxlingo.Utils.changeLanguage(langto, "shortName");
        }
        return ((langfrom=="af" && langto=="en") ||
                (langfrom=="as" && langto=="es") ||
                (langfrom=="ca" && langto=="es") ||
                (langfrom=="ca" && langto=="ro") ||
                (langfrom=="da" && langto=="sv") ||
                (langfrom=="de" && langto=="en") ||
                (langfrom=="en" && langto=="af") ||
                (langfrom=="en" && langto=="bn") ||
                (langfrom=="en" && langto=="we") ||
                (langfrom=="en" && langto=="de") ||
                (langfrom=="en" && langto=="ne") ||
                (langfrom=="en" && langto=="po") ||
                (langfrom=="es" && langto=="as") ||
                (langfrom=="es" && langto=="it") ||
                (langfrom=="es" && langto=="oi") ||
                (langfrom=="es" && langto=="oc") ||
                (langfrom=="es" && langto=="ro") ||
                (langfrom=="pe" && langto=="tg") ||
                (langfrom=="gd" && langto=="ir") ||
                (langfrom=="gl" && langto=="pt") ||
                (langfrom=="ic" && langto=="en") ||
                (langfrom=="it" && langto=="ca") ||
                (langfrom=="it" && langto=="es") ||
                (langfrom=="mk" && langto=="sb") ||
                (langfrom=="mk" && langto=="sh") ||
                (langfrom=="mk" && langto=="ss") ||
                (langfrom=="oi" && langto=="es") ||
                (langfrom=="oc" && langto=="es") ||
                (langfrom=="po" && langto=="en") ||
                (langfrom=="pt" && langto=="gl") ||
                (langfrom=="ro" && langto=="ca") ||
                (langfrom=="sc" && langto=="mk") ||
                (langfrom=="tg" && langto=="pe"));
    };
    
    pub.changeLanguagesToCode = function(language, translator) {
        switch (translator) {
            case "worldlingo":
                switch (language) {
                    case "cn": return "zh_cn";
                    case "tw": return "zh_tw";
                    default: break;
                }
                break;
            case "google": 
                switch (language) {
                    case "xx": return "auto"; 
                    case "cn": return "zh-CN"; 
                    case "tw": return "zh-TW"; 
                    case "bu": return "bg"; 
                    case "cr": return "hr"; 
                    case "cz": return "cs"; 
                    case "ph": return "tl"; 
                    case "he": return "iw"; 
                    case "ic": return "is"; 
                    case "in": return "id"; 
                    case "ir": return "ga"; 
                    case "ma": return "ms"; 
                    case "po": return "pl"; 
                    case "se": return "sr";
                    case "tu": return "tr";
                    case "we": return "cy"; 
                    case "pe": return "fa"; 
                    default: break;
                }
                break;
            case "bingtranslator":
                switch (language) {
                    case "xx": return ""; 
                    case "cn": return "zh-chs"; 
                    case "tw": return "zh-cht"; 
                    default: break;
                }
                break;
            case "freetranslation":
                switch (language) {
                    case "en": return 'English';
                    case "nl": return 'Dutch';
                    case "fr": return 'French';
                    case "de": return 'German';
                    case "it": return 'Italian';
                    case "no": return 'Norwegian';
                    case "pt": return 'Portuguese';
                    case "es": return 'Spanish';
                    default: break;
                }
                break;
            case "websphere":
                if (language=="pb")
                    return "bp";
                break;
                //falthrough
            case "gencat": 
                switch (language) {
                    case "en": return 'ENGLISH';
                    case "ca": return 'CATALAN';
                    case "fr": return 'FRENCH';
                    case "de": return 'GERMAN';
                    case "es": return 'SPANISH';
                    default: break;
                }
                break;
            case "gramtrans": 
                switch (language) {
                    case "en": return 'eng';
                    case "da": return 'dan';
                    case "eo": return 'epo';
                    case "no": return 'nor';
                    case "sv": return 'swe';
                    default: break;
                }
                break;
            case "intertran": 
                switch (language) {
                    case "en": return 'eng';
                    case "bu": return 'bul';
                    case "cr": return 'cro';
                    case "cz": return 'che';
                    case "da": return 'dan';
                    case "nl": return 'dut';
                    case "ph": return 'tag';
                    case "fi": return 'fin';
                    case "fr": return 'fre';
                    case "de": return 'ger';
                    case "el": return 'grk';
                    case "hu": return 'hun';
                    case "ic": return 'ice';
                    case "it": return 'ita';
                    case "ja": return 'jpn';
                    case "la": return 'ltt';
                    case "no": return 'nor';
                    case "po": return 'pol';
                    case "pt": return 'poe';
                    case "pb": return 'pob';
                    case "ro": return 'rom';
                    case "ru": return 'rus';
                    case "se": return 'sel';
                    case "sl": return 'slo';
                    case "es": return 'spa';
                    case "sa": return 'spl';
                    case "sv": return 'swe';
                    case "tu": return 'tur';
                    case "we": return 'wel';
                    default: break;
                }
                break;
            case "paktranslations":
                if (language=="ur")
                    return "urdu";
                break;
            case "promt": 
                switch (language) {
                    case "en": return 'e';
                    case "fr": return 'f';
                    case "de": return 'g';
                    case "it": return 'i';
                    case "pt": return 'p';
                    case "ru": return 'r';
                    case "es": return 's';
                    default: break;
                }
                break;
            case "yahoo":
                switch (language) {
                    case "cn": return "zh"; 
                    case "tw": return "zt";
                    default: break;
                }
                break;
            case "excite": 
                switch (language) {
                    case "ja": return 'JA';
                    case "en": return 'EN';
                    case "cn": return 'CH';
                    case "ko": return 'KO';
                    default: break;
                } 
                break;
            case "nifty":
                switch (language) {
                    case "ja": return '2';
                    case "en": return '1';
                    case "cn": return '6';
                    case "ko": return '9';
                    default: break;
                }
                break;
            case "vilnet":
                switch (language) {
                    case "en": return "e";
                    case "ja": return "j";
                    default: break;
                }
                break;
            case "translatica":
                if (language=="po")
                    return "pl";
                break;
            case "ask":  
                switch (language) {
                    case "en": return '1033';
                    case "fr": return '1036';
                    case "de": return '1031';
                    case "he": return '1037';
                    case "it": return '1040';
                    case "ja": return '1041';
                    case "pt": return '2070';
                    case "ru": return '1049';
                    case "es": return '1034';
                    default: break;
                }
                break;
            case "apertium":
                switch (language) {
                    case "as": return 'ast';                
                    default: break;
                }
                break;
            case "lingro":
                switch (language) {
                    case "en": return 'eng';
                    case "es": return 'spa';
                    case "fr": return 'fre';
                    case "it": return 'ita';
                    case "de": return 'ger';
                    case "po": return 'pol';
                    case "sw": return 'swe';
                    default: break;
                }
                break;
            default: break;
        }
        return language;
    };
    
    pub.backToOriginal = function(target) {
        var url = Foxlingo.Utils.getUrl("dontChangeCoding", target);
        if (!url)
            return '';
        
        try {
            if (!target)
                target = Foxlingo.Utils.getCurrentDocument();

            if (target.URL.indexOf("about:")!=-1 || !target.location || !target.location.hostname)
                return url;
            var hostname = target.location.hostname;
        }
        catch (err) {
            Foxlingo.Utils.debugException(err);
            return url;
        }
        
        if (!url)
            url = '';
        
        try {    
            url = fixUrl(url, hostname);
        }
        catch (err) {
            Foxlingo.Utils.debugException(err);
        }

        if (!url)
            url = '';

        return url;
    };
    
    pub.getTranslationUrl = function(translator, lang1, lang2, url, selection) {
        var newUrl = '';
        var pair;
        var s, s1, s2;
        
        if (!url && (!selection || selection.replace(/ /g, '')==='')) {
            newUrl = Foxlingo.Translation.Text.getEmptySelectionTextTranslationUrl(translator);
            if (newUrl)
                return newUrl;
        }

        if (url)
            s = url;
        else
            s = "http://www.foxlingo.com/showtext/index.html?text=" + selection;

        url = encodeURIComponent(url);
        selection = encodeURIComponent(selection);

        switch (translator) {
            case "apertium":
                if (url) {
                    s = "apertium-";
                    if (pub.isPairBeta(lang1, lang2, translator))
                        s += "unstable/tradurl.php?marcar=&inurl=" + url + "&direccion=";
                    else
                        s += "www/common/tradurl.php?mark=&inurl=" + url + "&dir="; 
                } else {
                    s = "webservice/ws.php?trad=unstable&text=" + selection + "&mode=";
                }
                return "http://xixona.dlsi.ua.es/" + s + lang1 + "-" + lang2;
            case "ask":
                return "http://uk.ask.com/fr?u=http%3A%2F%2Freverso2.com%2Fri45r%2FASP%2Furl%2Fresult.asp%3Fdirections%3D" + lang1 + "%24" + lang2 +
                    "%242%26autotranslate%3Don%26motsinconnus%3Dfalse%26templates%3DGeneral%26url%3D" + s +
                    "&s=uk&srcl=es&tarl=en&bu=http%3A%2F%2Fuk.ask.com%2Fweb%3Fq%3Dhola%26o%3D0%26l%3Ddir%26page%3D1%26dm%3Dlang%26advl%3D%26advc%3D" +
                    "&q=&o=0&l=dir&qt=&ac=24&dm=lang";
            case "cervantes":
                return "http://62.97.114.150/fstraducirpagina.aspx?slyidioma=" + pub.getGlobalPair(lang1, lang2, "cervantes") + "&url=" + s;
            case "citcat":
                if (!url)
                    return '';

                if (lang1=="en" && lang2=="ma")
                    pair = "1";
                else if (lang1=="ma" && lang2=="en")
                    pair = "2";
                else if (lang1=="en" && lang2=="cn")
                    pair = "3";
                return "http://www.citcat.com/cc?i=" + url + "&l=" + pair + "&t=u";
            case "elmundo":
                if (url)
                    s = "url/result_frame.asp?template=General&autotranslate=on&url=" + url + "&direction=";
                else
                    s = "textonly/default.asp?status=translate&source=" + selection + "&template=General&directions=";
                return "http://elmundo.reverso.net/" + s + pub.getGlobalPair(lang1, lang2, "elmundo");
            case "excite":
                if (url)
                    s = "/web/?&wb_lp=" + (lang1 + lang2).toUpperCase() + "&wb_dis=2&wb_url=" + url;
                else
                    s = '/?before=' + selection + '&wb_lp=' + lang1 + lang2 + '&start=%E7%BF%BB+%E8%A8%B3&';
                return "http://www.excite.co.jp/world/" + pub.getGlobalPair(lang1, lang2, "excite") + s;
            case "freetranslation":
                if (url) {
                    s = "fets5";
                    s1 = "&url=" + url;
                } else {
                    s = "ets";
                    s1 = "&mode=html&template=results_en-us.htm&srctext=" + selection + '&charset=UTF-8&J3YJDSG6LF9W6T6U';
                }
                return "http://" + s + ".freetranslation.com/?sequence=core&language=" + lang1 + "/" + lang2 + s1;
            case "gencat":
                if (url) {
                    s = "url";
                    s1 = "&url=" + url.replace(/\%20/g,"-");
                } else {
                    s = "text";
                    s1 = "&text=" + selection;
                }
                return "http://traductor.gencat.cat/url.do?CREATE_GLOSSARY=0&REQUEST_URL=http://traductor.gencat.cat/" + s +
                       ".do&MARK_COMPOUNDS=0&PPM_USE=1&REMOTE_ADDR=62.0.92.60&MARK_MEMORY=0&MARK_ALTERNATIVES=0&MARK_CONSTANTS=0" +
                       "&CREATE_CODING_LIST=0&MARK_UNKNOWNS=1&translationDirection=" +
                       lang1 + "-" + lang2 + s1 + "&subjectArea=(GV)";
            case "global":
                if (url) {
                    s = "geturl";
                    s1 = "&input_url=" + url;
                } else {
                    s = "texttrans.html";
                    s1 = "input_text=" + selection.replace(/ /g, '%2B');
                }
                return "http://babel.gts-translation.com/" + s + "?direction=" + pub.getGlobalPair(lang1, lang2, "global") + s1;
            case "google":
                s1 = "langpair=" + lang1 + "|" + lang2 + "&";
                if (url) {
                    s = "http://translate.google.com/translate?" + s1 + "u=" + url;
                } else {
                    s = 'http://www.google.com/translate_t?' + s1 + 'text=' + selection;
                }
                return s;
            case "gramtrans":
                s1 = lang1 + "2" + lang2;
                if (url) {
                    s = "http://gramtrans.com/gt/url/?url=" + url + "&pair=" + s1 + "&domain=&x-form-id=translate_url";
                } else {
                    s = 'http://gramtrans.com/?pair=' + s1 + '&input=' + selection;
                }
                return s;
            case "huajian":
                if (lang1=="en" && lang2=="cn")
                    pair = "01";
                else if (lang1=="cn" && lang2=="en")
                    pair = "02";
                else if (lang1=="ja" && lang2=="cn")
                    pair = "10";
                else if (lang1=="cn" && lang2=="ja")
                    pair = "03";
                return "http://220.194.61.120/cgi-bin/web.cgi?tranFlag=" + pair + "400&domain=00&url=" + s;
            case "imtranslator":
                if (url) {
                    s = "web";
                    s1 = "webtarget";
                    s2 = "b=&appid=&dic=&extsvr=&auto=&dirID=&dir=" + lang1 + "%2F" + lang2 + "&url=" + url;
                } else {
                    s = '';
                    s1 = "translator";
                    s2 = "text=" + selection + '&dir=' + lang1 + "/" + lang2;
                }
                return "http://" + s + "translation.paralink.com/" + s1 + ".asp?clientid=IDABXORB&afid=14645&" + s2;
            case "internostrum":
                return "http://internostrum.com/tradurl.php?linkdesc=&direccio=" + lang1 + "-" + lang2 + "&inurl=" + s;
            case "interpret":
                if (url)
                    translateAfrikaans(url, lang1, lang2);
                return '';
            case "intertran":
                if (url)
                    s = "url&Submit=Translate&url=" + url;
                else
                    s = "text&text=" + selection;
                return "http://www.tranexp.com:2000/InterTran?type=" + s + "&from=" + lang1 + "&to=" + lang2;
            case "lingro":
                if (url)
                    return "http://lingro.com/translate/" + url;
                else
                    return "http://lingro.com/" + "widgets/tinydict?dictionary=" + lang1 + "_" + lang2 + '&word=' + selection;
            case "bingtranslator":
                if (!url)
                    return '';

                return "http://www.microsofttranslator.com/BV.aspx?ref=BVNav&from=" + lang1 + "&to=" + lang2 + "&a=" + url;
            case "opentrad":
                if (!url)
                    s = s.replace(/%20/g, '_');
                return "http://www.opentrad.org/tradurl.php?marcar=&direccion=" + lang1 + "-" + lang2 + "&inurl=" + s;
            case "ocn":
                return "http://translate.ocn.ne.jp/LUCOCN/c3/hm_ex.cgi?SURL=" + s + "&XTYPE=1&SEARCH=T&SLANG=" + lang1 + "&TLANG=" + lang2;
            case "paktranslations":
                if (!url)
                    s = s.replace(/ /g, '_').replace(/%20/g, '_');
                return "http://paktranslations.com/?" + s;
            case "promt":
                return "http://www.online-translator.com/url/url/tran_url.asp?lang=en&direction=" + lang1 + lang2 + "&template=General&transliterate=on&autotranslate=on&url=" + s;
            case "sli":
                if (url) {
                    s = "navegador_en";
                    s2 = "&inurl=" + url;
                } else {
                    s = "tradtext";
                    s2 = '&cuadrotexto=' + selection;
                }
                return "http://sli.uvigo.es/tradutor/" + s + ".php?marcar=&direccion=" + lang1 + "-" + lang2 + s2;
            case "statmt":
                if (!url)
                    return "http://demo.statmt.org/web.cgi?url=" + s + selection + "&sysid=" + lang1 + "-" + lang2 + "-bare";
                return "http://demo.statmt.org/web.cgi?url=" + url + "&sysid=" + lang1 + "-" + lang2 + "-bare";
            case "toggletext":
                if (!url)
                    return '';

                return "http://www.toggletext.com/kataku.cgi?translate=web&frameset=1&input_url=" + url + "&langset_text=" + lang1 + "_" + lang2;
            case "traduku":
                if (url)
                    s = "u=" + url;
                else
                    s = "t=" + selection;
                return "http://traduku.net/cgi-bin/traduku?" + lang1 + "_" + lang2 + "&" + s;
            case "translatica":
                if (!url)
                    return '';

                return "http://www.translatica.pl/en/translating-portals/?urla=" + url + "&dir=" + lang1 + lang2;
            case "vilnet":
                if (!url)
                    return '';

                return "http://www.vil-net.jp/hon/cgi-bin/pgyaku.cgi?mod=" + lang1 + "2" + lang2 + "&url=" + url;
            case "websphere":
                return "http://192.84.47.104/demand?mtlang=" + lang1 + lang2 + "&translate=" + s;
            case "worldlingo":
                if (url)
                    return "http://www.worldlingo.com/S1790.5/translation?wl_srclang=" + lang1 + "&wl_trglang=" + lang2 + "&wl_url=" + url;
                return 'http://www.foxlingo.com/translate/index.html?' + 'langfrom=' + lang1 + '&langto=' + lang2 + '&text=' + selection;
            case "yahoo":
                if (url)
                    return "http://66.196.80.202/babelfish/translate_url_content?.intl=us&lp=" + lang1 + "_" + lang2 + "&trurl=" + url;
                return 'http://babelfish.yahoo.com/translate_txt?doit=done&intl=1&tt=urltext&trtext=' + selection + '&lp=' + lang1 + "_" + lang2;
            default: break;
        }
    };

    pub.getGlobalPair = function(lang1, lang2, translator) {
        var lang;
        var pair = '';

        switch (translator) {
            case "excite": 
                switch ((lang1 + lang2).toUpperCase()) {
                    case "ENJA":
                    case "JAEN": return "english";
                    case "CHJA":
                    case "JACH": return "chinese";
                    case "KOJA":
                    case "JAKO": return "korean";
                    default: break;
                }
                break;
            case "cervantes": 
                switch (lang1) {
                    case "en": 
                    case "fr": return lang1 + "_es";
                    case "ca": return "catesp";
                    case "pb": return "braesp";
                    case "gl": return "galesp";
                    case "es":
                        switch (lang2) {
                            case "gl": return "espgal";
                            case "en": return "es_en";
                            case "fr": return "es_fr";
                            case "ca": return "espcat";
                            case "pb": return "espbra";
                            default: break;
                        } 
                        break;
                    default: break;
                }
                break;
            case "interpret" :
                for each (lang in [lang1, lang2]) {
                    switch (lang) {
                        case "en": pair += "eng"; break;
                        case "de": pair += "dt"; break;
                        case "se": pair += "sp"; break;
                        case "sv": pair += "sw"; break;
                        default: pair += lang; break;
                    }
                    if (lang==lang1)
                        pair += "to";
                }
                break;
            case "elmundo":
                switch (lang1 + "-" + lang2) {
                    case 'en-es': return '2097153';
                    case 'es-en': return '65568';
                    case 'fr-es': return '2097160';
                    case 'es-fr': return '524320';
                    case 'en-de': return '262145';
                    case 'de-en': return '65540';
                    default: break;
                }
                break;
            default: break;
        }
        return pair;
    };
        

    //private functions
    function findTranslateRequest(urlToFind) {
        for each (var item in mTSQueue) {
            if (item.url==urlToFind)            
                return true;
        }
        return false;
    }
    
    function doesTranslatorSupportLanguages(translator, langfrom, langto) {
        var hasLangFrom = false;
        var hasLangTo = false;

        if (Foxlingo.Prefs.getBoolPref("autotransWorldlingo") && (translator=="google" || translator=="intertran")) {
            var languages = pub.getTranslatorArray(translator);
            return (((translator=="google" && langfrom=="auto") || languages.indexOf(langfrom)!=-1) && languages.indexOf(langto)!=-1);
        }
        switch (translator) {
            case "worldlingo": if (Foxlingo.Prefs.getBoolPref("autotransWorldlingo")) {
                                   var languages = pub.getTranslatorArray(translator); 
                                   return ((languages.indexOf(langfrom)!=-1) && languages.indexOf(langto)!=-1 && langfrom!=langto);
                               } else {
                                   return ((langfrom!="ha" && langto=="ha") || (langto!="ha" && langfrom=="ha") ||
                                                   (langfrom!="ma" && langto=="ma") || (langto!="ma" && langfrom=="ma") ||
                                                   (langfrom!="ps" && langto=="ps") || (langto!="ps" && langfrom=="ps") ||
                                                   (langfrom!="pe" && langto=="pe") || (langto!="pe" && langfrom=="pe") ||
                                                   (langto!="so" && langfrom=="so") ||
                                                   (langfrom!="ur" && langto=="ur") || (langto!="ur" && langfrom=="ur"));
                               }
            case "traduku":
                return ((langfrom=="eo" && langto=="hu") ||
                        (langfrom=="eo" && langto=="en") ||
                        (langto=="eo" &&
                         (langfrom=="en" || langfrom=="nl" || langfrom=="fr" || langfrom=="de" || langfrom=="hu" ||
                          langfrom=="pt" || langfrom=="es" || langfrom=="sv"))); 
            case "apertium":
                return ((langfrom=="oi" && langto=="ca") || 
                        (langfrom=="ca" && langto=="oi") || 
                        (langfrom=="oc" && langto=="ca") || 
                        (langfrom=="ca" && langto=="oc") || 
                        (langfrom=="ca" && langto=="eo") || 
                        (langfrom=="es" && langto=="eo") || 
                        (langfrom=="br" && langto=="fr") || 
                        (langfrom=="es" && langto=="va") || 
                        (langfrom=="eu" && langto=="es") || 
                        (langfrom=="fa" && langto=="tg") || 
                        (langfrom=="gd" && langto=="ir") || 
                        (langfrom=="ic" && langto=="en") || 
                        (langfrom=="no" && langto=="nn") || 
                        (langfrom=="nn" && langto=="no") || 
                        (langfrom=="tg" && langto=="fa") || 
                        //beta
                        (langfrom=="as" && langto=="es") ||
                        (langfrom=="ca" && langto=="es") ||
                        (langfrom=="ca" && langto=="ro") ||
                        (langfrom=="en" && langto=="bn") ||
                        (langfrom=="en" && langto=="ne") ||
                        (langfrom=="es" && langto=="as") ||
                        (langfrom=="es" && langto=="oi") ||
                        (langfrom=="es" && langto=="oc") ||
                        (langfrom=="pe" && langto=="tg") ||
                        (langfrom=="gd" && langto=="ir") ||
                        (langfrom=="it" && langto=="ca") ||
                        (langfrom=="mk" && langto=="sb") ||
                        (langfrom=="mk" && langto=="sh") ||
                        (langfrom=="mk" && langto=="ss") ||
                        (langfrom=="oi" && langto=="es") ||
                        (langfrom=="oc" && langto=="es") ||
                        (langfrom=="pt" && langto=="gl") ||
                        (langfrom=="ro" && langto=="ca") ||
                        (langfrom=="sc" && langto=="mk") ||
                        (langfrom=="tg" && langto=="pe"));
            default: return false;
        }
    } 
    
    function fixUrlPart1(url, hostname) {
        var originalUrl = url;
        var isFound = false;
        var urlTermNames = getUrlTermNamesByHostname(hostname);
        if (urlTermNames.length===0)
            return originalUrl;

        var parts = url.split("?");
        if (!parts || !parts[1])
            return originalUrl;

        url = parts[1];
        for each (var term in url.split('&')) {
            parts = term.split("=");
            if (!parts.length==2)
                continue;
            for each (var urlTermName in urlTermNames) {

                if (parts[0]==urlTermName)
                    return parts[1];
            }
        }
        return originalUrl;
    }

    function fixUrl(url, hostname) {
        var tempItem;
        var originalUrl = url;
        url = fixUrlPart1(url, hostname);
     
        switch (hostname) {
            case 'www.worldlingo.com':
                if (originalUrl!=url) {
                    url = url.replace(/wl_method%3Dget/gi, "");
                    if (url.indexOf("ixquick.com")!=-1 && url.lastIndexOf("args%253D")!=-1)  
                        url = 'http://' + url.substr(url.lastIndexOf("args%253D") + 9); 
                }
                break;
            case 'traductor.gencat.cat':
                if (originalUrl!=url)
                    url = url.replace(/\\1/g,"?");
                break;
            case 'interpret.co.za':
                tempItem = document.getElementById("formurl");
                if (!tempItem)
                    break;

                if (tempItem.url.value)
                    url = tempItem.url.value;
                else if (window.opener) {
                    tempItem = window.opener.document.getElementById("formurl");
                    if (tempItem && tempItem.url.value)
                        url = tempItem.url.value;
                }
                break;
            case 'lingro.com':
                url = url.replace(/http%3A%2F%2Flingro.com%2Ftranslate%2F/g,"");
                tempItem = document.getElementById("lingroWebForm");
                if (!tempItem)
                    break;

                if (tempItem.translate_box.value)
                    url = tempItem.translate_box.value;
                else if (window.opener) {
                    tempItem = window.opener.document.getElementById("lingroWebForm");
                    if (tempItem && tempItem.translate_box.value)
                        url = tempItem.translate_box.value;
                }
                break;
            case 'translate.ocn.ne.jp':
                url = url.replace(/http%3A%2F%2Ftranslate.ocn.ne.jp%2FLUCOCN%2Fc3%2Fhm.cgi%2F/g,"");
                url = url.replace(/&SLANG%3D*.*&TLANG%3D*.*&XTYPE%3D0&XCHARSET%3DISO-8859-1/g,"");
                break;
            case 'gramtrans.com':
                url = url.replace(/http%3A%2F%2Fgramtrans.com%2Fgt%2Furl%2F%3Furl%3D/g,"");
                url = url.replace(/&pair%3D*.*&domain%3D&x-form-id%3Dtranslate_url/g,"");
                break;
            case 'demo.statmt.org':
                url = url.replace(/http%3A%2F%2Fdemo.statmt.org%2Fweb.cgi%3Furl%3D/g,"");
                url = url.replace(/&sysid%3D*.*/g,"");
                break;
            default: break;
        }
//        url = url.replace(/\%3D/ig,"=").replace(/\%3F/ig,"?");
//        url = url.replace(/\%3A/ig,":").replace(/\%2F/ig,"/");
        url = decodeURIComponent(url);
        return url;
    }

    function getUrlTermNamesByHostname(hostname) {
        switch (hostname) {
            case 'www.worldlingo.com': return ["wl_url"];
            case 'translate.google.com':
            case 'www.google.com':
            case '66.249.93.104':
            case '74.125.93.104':
            case '74.125.19.102':
            case 'traduku.net':
            case '64.233.179.104': return ["u"]; 
            case '66.196.80.202': return ["trurl", "url"];
            case 'www.online-translator.com':
            case 'webtranslation.paralink.com':
            case 'traductor.gencat.cat':
            case 'www.tranexp.com':
            case 'paktranslations.com':
            case 'interpret.co.za':
            case "uk.ask.com":
            case '62.97.114.150':
            case 'nifty.amikai.com':
            case 'alice-group.amikai.com':
            case 'rinkworks.com':
            case 'elmundo.reverso.net':
            case 'www.vil-net.jp':
            case '220.194.61.120':
            case 'www.translatica.pl':
            case 'fets5.freetranslation.com': return ["url"];
            case '216.109.124.98':
            case '72.30.186.56':
            case 'babelfish.yahoo.com':
            case '74.6.146.244': return ["trurl"]; 
            case '192.195.29.104': return ["translate"];
            case 'www.toggletext.com':
            case 'babel.gts-translation.com': return ["input_url"];
            case 'www.microsofttranslator.com': return ["a"];
            case 'www.opentrad.org':
            case 'sli.uvigo.es':
            case 'xixona.dlsi.ua.es': 
            case 'internostrum.com': return ["inurl"];
            case 'translate.ocn.ne.jp': return ["SURL"];
            case 'www.excite.co.jp': return ["wb_url"];
            case 'www.citcat.com': return ["i"];
            default: return [];
        }
    }


    function translateAfrikaans(newUrl, lang1, lang2) {
        var tempItem = document.getElementById("formurl");

        tempItem.target = Foxlingo.Utils.getTarget();
        if (lang1=='en') {
            tempItem.from.value = 'en';
            tempItem.to.value = 'af';
        } else {
            tempItem.from.value = 'af';
            tempItem.to.value = 'en';
        }
        tempItem.url.value = newUrl;
        tempItem.submit();    
    }
    
    return pub;
}();
